/***** Maxim Generalov,  09-2009 *********************************/
/*  fat.c fat-12 for mmc card tested on 16MB. It use             */
/*  mmc.c, mmc.h by Rolf Freitag 5/2003                          */
/*****************************************************************/
/* Version 2 : added processing for hidden sectors 17/09/2016   */
/* Version 2.1 : Patched end of write at '\0' symbol 30/09/2016                                                             */
/*****************************************************************/

#include "fat32.h"
#include "mmc.h"
#include "gps.h"
#include "stdio.h"
#include "string.h"





int fileSearch(char * file_name);
int freeClustSearch( int StartCluster );
int fileCreate( char * file_name);
int markCluster(int cluster_number, unsigned claster_value);
int clValue(int cluster_number);
int catPlace(int id, tCAT * c);


// external buffer which is use to read/write in MMC card
extern char mmc_buffer[512];
extern struct CURRENT_TIME ctime;
struct tFILE fp[10];

//char tmp_buffer[512];
char fat_buffer[512];
int used_fat_sector=-1; 
struct BOOT_SECT b;
struct tCAT dir;
//unsigned int CatSect=0; 
//unsigned int CatClust=0; 

/*DEBUG***********************************/
//int w0=0, w1=0;
/*DEBUG***********************************/

int fatInit(void)
{   int i;
 // Search and Read first Block back to buffer
    memset(&mmc_buffer,0x00,512);
//    mmcReadBlock(0,512);
    
    
    for(i = 0; i < 65535 ; i++)
    {  
      if(mmcReadBlock(i << 9,512) != 0) return(-1);
      if((mmc_buffer[510] == 0x55) &&
      (mmc_buffer[511] == 0xAA) &&
      (mmc_buffer[82] == 0x46) &&
      (mmc_buffer[83]  == 0x41) &&
      (mmc_buffer[84]  == 0x54) &&
      (mmc_buffer[85]  == 0x33) &&
      (mmc_buffer[86]  == 0x32))  
      {
         //mmc_buffer[28] = 0;
         //mmcWriteBlock(0);    
        
        b.SectSize   = mmc_buffer[11] | (mmc_buffer[12]<<8); 
        b.ClustSize  = mmc_buffer[13];
        b.HiddenSecs = mmc_buffer[28] | (mmc_buffer[29]<<8) | (mmc_buffer[30]<<16) | (mmc_buffer[31]<<24);
        b.ResSecs    = (mmc_buffer[14] | (mmc_buffer[15]<<8))+b.HiddenSecs; 
        b.FatCnt     = mmc_buffer[16]; 
        b.TotSecs    = mmc_buffer[32] | (mmc_buffer[33]<<8) | (mmc_buffer[34]<<16) | (mmc_buffer[35]<<24); 
        b.FatSize    = mmc_buffer[36] | (mmc_buffer[37]<<8) | (mmc_buffer[38]<<16) | (mmc_buffer[39]<<24); 
        b.RootClust  = mmc_buffer[44] | (mmc_buffer[45]<<8) | (mmc_buffer[46]<<16) | (mmc_buffer[47]<<24); 
        b.RootCat    = b.ResSecs + (b.FatSize*b.FatCnt);//+b.HiddenSecs;
        b.StartData  = b.RootCat;
	b.RootSize   = b.ClustSize; 
		
        if(i ==  (int)mmc_buffer[28]) break;
        return(-1);        
      }
  }   
  return(b.HiddenSecs);
}

int fileSearch(char * file_name)
{
  int k, i, j, fptr, CatSect, CatClust;
  CatSect = b.RootCat;
  CatClust = b.RootClust;
  j = 0;
  while( CatClust <= 0x0FFFFFF8 )
  { 
	for(k = 0; k < b.RootSize; k++)            
	{ 

          mmcReadBlock((CatSect+k)*b.SectSize,512);
	  for(i=0; i < 16; i++)
	  {
	    fptr=i*32;
	    if((mmc_buffer[fptr] != 0xE5) && (mmc_buffer[fptr] !=0) &&
	    ((mmc_buffer[fptr+0xb]&0x18) == 0))
	    {
              if( strncmp( file_name, &mmc_buffer[fptr], 11) == 0 ) 
              return(i+(k*16)+ (j*128));
	    }
	  }
	}
  CatClust = clValue(CatClust);
  CatSect =  b.RootCat + ((CatClust - b.RootClust)<<3);
  j++;
  }
  
    return(-1);
}
 
int clValue(int cluster_number)
{ unsigned k, n, iClust, vClust;
  extern int used_fat_sector;

   iClust = cluster_number << 2;

   k=iClust>>9;
   
   iClust = iClust - (k<<9);
   if( k != used_fat_sector)
   {
      mmcReadBlock((b.ResSecs+k)*b.SectSize,512);
      for (n=0; n < 512; n++) fat_buffer[n] = mmc_buffer[n]; 
      used_fat_sector = k;
   }


      vClust = (unsigned)(fat_buffer[iClust] | (fat_buffer[iClust + 1] << 8) |
        (fat_buffer[iClust+2] << 16) | (fat_buffer[iClust + 3] << 24));


return(vClust);
}

int freeClustSearch( int StartCluster )
{  
   int  i; 

   for(i=StartCluster; i < (b.TotSecs/b.ClustSize); i++)
   {
     if(clValue(i) == 0)
       return(i);
   }

return(-1);

}

int fileCreate( char * file_name)
{
  int k, i, j, fptr, CatSect, CatClust;
  CatSect = b.RootCat;
  CatClust = b.RootClust;
  j = 0;
  while(CatClust < 0x0FFFFFF8)
  { 
	for(k = 0; k < b.RootSize; k++)            
	{ 

        mmcReadBlock((CatSect+k)*b.SectSize,512);
	  for(i=0; i < 16; i++)
	  {
	    fptr=i*32;
            if((mmc_buffer[fptr] == 0xE5) || (mmc_buffer[fptr] ==0)) 
            {
              strncpy( &mmc_buffer[fptr], file_name, 11 );
              mmc_buffer[fptr+0x0B] = 0x20;
              mmc_buffer[fptr+0x0C] = 0x0;
              mmc_buffer[fptr+0x0D] = 0x43;
              mmc_buffer[fptr+0x0E] =  ((ctime.hour << 11) | ( ctime.minute << 5 ) | ctime.secunde/2) & 0xFF; 
              mmc_buffer[fptr+0x0F] =  (((ctime.hour << 11) | ( ctime.minute << 5 ) | ctime.secunde/2) & 0xFF00)>>8;
              mmc_buffer[fptr+0x10] =  (((ctime.year-1980) << 9) | ( ctime.month << 5 ) | ctime.day) & 0xFF; 
              mmc_buffer[fptr+0x11] =  ((((ctime.year-1980) << 9) | ( ctime.month << 5 ) | ctime.day) & 0xFF00)>>8;
              mmc_buffer[fptr+0x12] =  (((ctime.year-1980) << 9) | ( ctime.month << 5 ) | ctime.day) & 0xFF; 
              mmc_buffer[fptr+0x13] =  ((((ctime.year-1980) << 9) | ( ctime.month << 5 ) | ctime.day) & 0xFF00)>>8;
              mmc_buffer[fptr+0x14] = 0x0;
              mmc_buffer[fptr+0x15] = 0x0;
              mmc_buffer[fptr+0x16] =  ((ctime.hour << 11) | ( ctime.minute << 5 ) | ctime.secunde/2) & 0xFF; 
              mmc_buffer[fptr+0x17] =  (((ctime.hour << 11) | ( ctime.minute << 5 ) | ctime.secunde/2) & 0xFF00)>>8;
              mmc_buffer[fptr+0x18] =  (((ctime.year-1980) << 9) | ( ctime.month << 5 ) | ctime.day) & 0xFF; 
              mmc_buffer[fptr+0x19] =  ((((ctime.year-1980) << 9) | ( ctime.month << 5 ) | ctime.day) & 0xFF00)>>8;
              mmc_buffer[fptr+0x1A] =  0; 
              mmc_buffer[fptr+0x1B] =  0;
              mmc_buffer[fptr+0x1C] =  0; 
              mmc_buffer[fptr+0x1D] =  0;
              mmc_buffer[fptr+0x1E] =  0; 
              mmc_buffer[fptr+0x1F] =  0;
              mmcWriteBlock((CatSect+k)*b.SectSize); //writing modified catalogue
              return(i+(k*16)+ (j*128));
            }
	  }
	}
  CatClust = clValue(CatClust);
  CatSect =  b.RootCat + ((CatClust - b.RootClust)<<3);
  j++;
  }
return(-2);
}

int markCluster(int cluster_number, unsigned cluster_value)
{
   int n, k, iClust; 

   iClust = cluster_number << 2;

   k=iClust>>9;
   
   iClust = iClust - (k<<9);
   if( k != used_fat_sector)
   {
      mmcReadBlock((b.ResSecs+k)*b.SectSize,512);
      for (n=0; n < 512; n++) fat_buffer[n] = mmc_buffer[n]; 
      used_fat_sector = k;
   }   
  
   *(unsigned*)(&fat_buffer[iClust]) = cluster_value;
   

      for (n=0; n < 512; n++) mmc_buffer[n] = fat_buffer[n]; 
      mmcWriteBlock((b.ResSecs+k)*b.SectSize); //wirting fat1
      mmcWriteBlock((b.ResSecs+k+b.FatSize)*b.SectSize); //wirting fat2
//      for (n=0; n < 512; n++) mmc_buffer[n] = tmp_buffer[n]; 
          
return(0);
  
}

tFILE * fileOpen(char * file_name, char mode)
{
  int n, vClust, tmp_word, fptr, cluster_number, CatSect;
  tFILE * f;
//  extern struct tFILE fp[10];
//  f=(tFILE*) (&fp[0]);
  
  tmp_word=fileSearch(file_name);
  if( mode == 'c')
  {
    if( tmp_word >= 0) return(NULL);
    else tmp_word = fileCreate(file_name);
  }
  else  if( tmp_word < 0)  return(NULL);
  for(n=0; n < 10; n++) 
  {
    if( fp[n].id == 0 ) break;
  }
   f=&fp[n];
   f->id = tmp_word;
/*
   j = tmp_word/128;    // 128 = (b.SectSize*b.ClustSize)/32
   k = (tmp_word%128)/16; // 16 = 512/32
   i = tmp_word - j*128 - k*16; 
 
   vClust = b.RootClust;
   for( n=0 ; n < j; n++)
   {
      vClust = clValue(vClust);
   }
   CatSect = (b.RootCat+(vClust-b.RootClust)*b.ClustSize+k);

   fptr = i<<5;
*/ 

   catPlace(tmp_word, &dir);
   CatSect = dir.catSect;
   fptr = dir.fptr;   
   
   mmcReadBlock(CatSect*b.SectSize,512);
   
   f->size=(mmc_buffer[fptr+0x1F]<<24) | (mmc_buffer[fptr+0x1E]<<16) |
      (mmc_buffer[fptr+0x1D]<<8) | mmc_buffer[fptr+0x1C];
   f->current_cluster=f->first_cluster = mmc_buffer[fptr+0x1A] | (mmc_buffer[fptr+0x1B]<<8) | 
     (mmc_buffer[fptr+0x14]<<16) | (mmc_buffer[fptr+0x15]<<24); 
   f->current_sector=f->first_sector  = b.StartData + ((f->first_cluster-2)*b.ClustSize);
   f->current_offset=0;
   cluster_number = f->first_cluster;
   tmp_word=f->size;
  while(1)
  {
          vClust = clValue(cluster_number);       
    
          if(vClust >= 0x0FFFFFF8)
          {
            f->last_cluster = cluster_number;
            break;
          }
          else if(vClust == 0x0)
          {
            return(NULL);
          }
          else
          {
            cluster_number = vClust;
            tmp_word-=b.ClustSize*b.SectSize;
            if(tmp_word < 0)
            {
              return(NULL);
            }
          }
  }      
   f->last_sector = (b.StartData + ((f->last_cluster-b.RootClust)*b.ClustSize))
     +tmp_word/b.SectSize;
   f->last_offset = tmp_word%b.SectSize;
return(f);  
}

int fileRead(tFILE * f, char * string, int bytes)
{
  int i, k, next_cluster;  
  
  if((f->current_sector == f->last_sector) &&
     (f->current_offset >= f->last_offset))
    return(-1);
  
     mmcReadBlock(f->current_sector*b.SectSize,512);
    
    for( i=0, k=f->current_offset; i < bytes; i++)
    {
      if(k > 511)
      {
        f->current_sector++;
        if( ((f->current_sector-b.StartData)%b.ClustSize) == 0)
        {
           next_cluster=clValue(f->current_cluster);
           if (next_cluster >=0x0FFFFFF8)  return(i);
           else f->current_cluster = next_cluster;
           f->current_sector = b.StartData + ((f->current_cluster-b.RootClust)*b.ClustSize);
        }
        mmcReadBlock(f->current_sector*b.SectSize,512);
        k=0;
        f->current_offset =0;
      }
      string[i] = mmc_buffer[k];
      k++;
      if(string[i] == 0x0A)
      {
//        w0++;
        break; //break on end string
      }
      if(f->current_sector == f->last_sector)
      {
        if(k== f->last_offset)
        {
        break; // break on end of file
        }
      }
    }
    f->current_offset=k;
    return(i);
}


int fileWrite(tFILE * f, char * string, int bytes)
{
  int i, k, fptr, next_cluster;  
  int CatSect;

     catPlace(f->id, &dir);
     CatSect = dir.catSect;
     fptr = dir.fptr;
  
  
    if(f->size == 0)
    {
      next_cluster=freeClustSearch(2);
      if(next_cluster <= 0)
      {
        return(0);
      }
      else
      {
        markCluster(next_cluster, 0x0FFFFFFF);

        f->first_cluster = f->last_cluster = f->current_cluster = next_cluster;      
        f->last_sector  = f->current_sector = b.StartData + ((f->current_cluster-b.RootClust)*b.ClustSize);
        f->first_sector = f->last_sector;
        f->last_offset  = f->current_offset = 0;
        
        
/*        t = f->id;
        j = t/128;    // 128 = (b.SectSize*b.ClustSize)/32
        k = (t%128)/16; // 16 = 512/32
        fptr = (t - j*128 - k*16)<<5;
 
        vClust = b.RootClust;
        for( n=0 ; n < j; n++)
        {
          vClust = clValue(vClust);
        }

        CatSect = b.RootCat+(vClust-b.RootClust)*b.ClustSize+k;
*/
       
        mmcReadBlock(CatSect*b.SectSize,512);
        
        mmc_buffer[fptr+0x1A] = (f->first_cluster & 0xff);
        mmc_buffer[fptr+0x1B] = ((f->first_cluster >> 8) & 0xff);
        mmc_buffer[fptr+0x14] = ((f->first_cluster >> 16) & 0xff);
        mmc_buffer[fptr+0x15] = ((f->first_cluster >> 24) & 0xff);
        mmcWriteBlock(CatSect*b.SectSize);
      } 
    }
    mmcReadBlock(f->current_sector*b.SectSize,512);

    for( i=0, k=f->current_offset; i < bytes; i++)
    {
      if(k > 511)
      {
        mmcWriteBlock(f->current_sector*b.SectSize); //wirting sector
        f->current_sector++;
        if(((f->current_sector-b.StartData)%b.ClustSize) == 0)
        {
          
          next_cluster=freeClustSearch(f->current_cluster);
          if(next_cluster < 0)
          {
            f->size = f->size+i;
            f->last_offset=f->current_offset=k;
            f->last_sector=f->current_sector;
            f->last_cluster=f->current_cluster;
            goto exit; 
          }
          else
          {
            markCluster(f->current_cluster,next_cluster);
            markCluster(next_cluster, 0x0FFFFFFF);
          }
          f->current_cluster = next_cluster;
          f->current_sector = b.StartData + ((f->current_cluster-b.RootClust)*b.ClustSize);
        /*   next_cluster=clValue(f->current_cluster);
           if (next_cluster >=0xFF8)  return(i);
           else f->current_cluster = next_cluster;*/
        }
        mmcReadBlock(f->current_sector*b.SectSize,512);
        k=0;
      }
      mmc_buffer[k] = string[i];
      if(string[i] == 0x0)
      {
//        w1++;
        break;
      }
      k++;
    }
    mmcWriteBlock(f->current_sector*b.SectSize); //wirting sector
exit:
//    f->size = f->size+i+1;
    f->size = f->size+i;
    f->last_offset=f->current_offset=k;
    f->last_sector=f->current_sector;
    f->last_cluster=f->current_cluster;
    // modification catalogue!!
   mmcReadBlock(CatSect*b.SectSize,512);
   mmc_buffer[fptr+0x16] =  ((ctime.hour << 11) | ( ctime.minute << 5 ) | ctime.secunde/2) & 0xFF; 
   mmc_buffer[fptr+0x17] =  (((ctime.hour << 11) | ( ctime.minute << 5 ) | ctime.secunde/2) & 0xFF00)>>8;
   mmc_buffer[fptr+0x18] =  (((ctime.year-1980) << 9) | ( ctime.month << 5 ) | ctime.day) & 0xFF; 
   mmc_buffer[fptr+0x19] =  ((((ctime.year-1980) << 9) | ( ctime.month << 5 ) | ctime.day) & 0xFF00)>>8;
   mmc_buffer[fptr+0x1C] =  f->size & 0x000000FF; 
   mmc_buffer[fptr+0x1D] = (f->size & 0x0000FF00)>>8;
   mmc_buffer[fptr+0x1E] = (f->size & 0x00FF0000)>>16;  
   mmc_buffer[fptr+0x1F] = (f->size & 0xFF000000)>>24;
   mmcWriteBlock(CatSect*b.SectSize);
   return(i);
}

void fileReset(tFILE * f)
{
   f->current_cluster=f->first_cluster; 
   f->current_sector=f->first_sector;
   f->current_offset=0;
}
 
int catPlace(int id, tCAT * c)
{
  int n, vClust, CatClustNum, CatSectNum; 

    CatClustNum = id/128;    // 128 = (b.SectSize*b.ClustSize)/32
    vClust = b.RootClust;
    for( n=0 ; n < CatClustNum; n++)
    {
      vClust = clValue(vClust);
    }
    
    CatSectNum = (id%128)/16; // 16 = 512/32
    c->fptr = (id - CatClustNum*128 - CatSectNum*16)<<5;
    c->catSect = b.RootCat+(vClust-b.RootClust)*b.ClustSize+CatSectNum;
    return(vClust);
}
