/*****************************************************************/
/***** Maxim Generalov,  10-2010 *********************************/
/*****************************************************************/
//*--------------------------------------------------------------------------------------
//*      ATMEL Microcontroller Software Support  -  ROUSSET  - 
//*--------------------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*--------------------------------------------------------------------------------------
//* 2.0 02/Sep/09 second release of xpilot
//* 2.1 04/Sep/09 second release of xpilot TC2 patched edges because changed polarity pulses from receiver..
//* 2.2 06/Sep/09 second release of xpilot WDTIMER added..
//* 2.3 16/Sep/09 second release of xpilot speed calculation added (sqrt*5.2), calibrated
//* 2.4 19/Sep/09 second release of xpilot vgyro meassured and calibrated
//* 2.5 19/Sep/09 second release of xpilot I insert really asin()  to all formuls  
//* 2.6 20/Sep/09 second release of xpilot I insert gyro compensation for 4 min  
//* 2.7 25/Okt/09 second release of xpilot with MEV-50A  
//* 2.8 30/Sep/10 second release of xpilot changed fss function and correcting vgyro and speed; 
//* 2.9 01/Okt/10 akkumulator monitor implemented = v_akk
//* 2.10 06/Okt/10 DMA on UART implemented, TWI debuged... 
//* 2.11 25/Okt/10 Defined number of channels for RC-control system. It is NUMCHAN. I will use Dragun Sportster also.
//* 2.12 04/Nov/10 For electrical motor I set start throttle to 0.
//* 2.13 08/Nov/10 The soft-int by IRQ1 is added, that MMC card does not make a problems to real time
//* 2.14 14/Sep/16 The GPS inserted
//* 2.15 18/Sep/16 Hidden secs for SD card processed
//* 2.16 14/Jun/17 The method for restoring the order of the channel impulses was changed. Hardware error was debugged:
//  The emitter junction of the transistor is shunted by resistor 1kOm to exclude false pulse received by controller
//  for high impedance input.
//* 2.17 28/Jun/17 FlySky mode for reciver signals added.
//* 2.18 21/Jun/17 GPS problem dabtint debugged. 
//* 2.19 28/Sep/17 Kfar_speed added. 
//*--------------------------------------------------------------------------------------
#define  VERSION    "\r\n*** GPILOT V2.19.0 *** 28/Sep/2017\r\n"

//#define FLYSKY 
#define DRAGON_SPORTSTER 
//#define  DUSTY  


#include "Board.h" 
#include "usart.h"  
#include "twi.h"
#include "gps.h"
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

//#define WAIT_GPS
//#define ONLY_THROTTLE            // only  throttle is controlled by pult
#define MMC_CARD
#define COMPENSATION              
//#define NO_SIN
#define EXEL_COMPATIBLE_PRINT
//#define APILOT_INFO_PRINT               
//#define TRACKING_INFO_PRINT               
//#define GPS_INFO_PRINT         
#define ROLL_BY_GYRO           
//#define ROLL_DEBUG            // we set speed equal 30 km/h and we don't meassure it;           
//#define AUTO_ROLL           //roll from 10 km/h will static from accelerometers, more then 10 - from gyro
//#define WATCH_DOG

#ifdef MMC_CARD
#include "mmc.h"
#include "fat.h"
#endif

#define PI 3.141592654
#define LEDON  *AT91C_PIOA_CODR = AT91C_PIO_PA8;  //set reg to 0 (led on)
#define LEDOFF *AT91C_PIOA_SODR = AT91C_PIO_PA8;  //set reg to 1 (led off)

#define TPOFF    *AT91C_PIOA_CODR = AT91C_PIO_PA7;  //set reg to 0 
#define TPON     *AT91C_PIOA_SODR = AT91C_PIO_PA7;  //set reg to 1 

#define GPEOFF    *AT91C_PIOA_CODR = AT91C_PIO_PA19;  //set reg to 0 
#define GPEON     *AT91C_PIOA_SODR = AT91C_PIO_PA19;  //set reg to 1 


#define WDRESET   *AT91C_WDTC_WDCR=0xA5000001; //reset watch dog timer
  

#define WDUTY    740   //150
#define WIDE     1460  //571
#define WPAUSE   15060 //3940
#define WMAX     3100//3100   
#define WMIN     1300//1300
#define WMED     (WMAX + WMIN)/2
#define WAMP     (WMAX - WMIN)/2

#ifdef DRAGON_SPORTSTER
#define NUMCHAN  6 //for Dragun Sportster
#elif  DUSTY 1 
#define NUMCHAN  8 //for Dusty
#else
#define NUMCHAN  8 //for Super Frontier
#endif

#define GPS_MIN_SPEED 10
#define RadRazv       50
#define FAR_SAMPLES   128
#define FSS_SAMPLES   4
#define ALT_SAMPLES   64
#define SPEED_SAMPLES 64
#define SPEED_SAMPLES_HPF 256 //HPF = Hihg Ptiority Function
#define GYRO_SAMPLES  8
#define AP2_SAMPLES   4
#ifdef DRAGON_SPORTSTER
  #define TROTTLE_XX    0     //for Dragun Sportster
#elif DUSTY 1
  #define TROTTLE_XX    0     // Dusty
#else
  #define TROTTLE_XX    50    //for Super Frontier
#endif

//MAIN POINTER
AT91PS_PIO    m_pPio   = AT91C_BASE_PIOA;
AT91PS_PDC    m_pPDC0  = AT91C_BASE_PDC_US0;
AT91PS_PDC    m_pPDC1  = AT91C_BASE_PDC_US1;
AT91PS_PMC    m_pPMC   = AT91C_BASE_PMC;


typedef struct FSSi
{
  short int pitch;
  short int roll;
  short int speed;
  short int altitude;
  short int heading;
}FSSi;
typedef struct FSSo
{
  short int ailerons;
  short int elevator;
  short int rudder;
  short int throttle;
}FSSo;
typedef struct FSSp
{
  short int pitch;
  short int roll;
  short int speed;
}FSSp;

typedef struct FSSc
{
  float Kst_roll;
  float Kast_roll;
  float Kfar_roll;
  float Kst_pitch;
  float Kast_pitch;
  float Kfar_pitch;
  float Kst_speed;
  float Kast_speed;
  float Kfar_speed;
  float Kst_hdg;
  float Kast_hdg;
  float Kst_alt;
  float Kast_alt;
}FSSc;

typedef struct WPS //way point structure
{
  int point_num;
  int alt_requested;
  int head_requested;
  int speed_requested;
  int time_requested;
  int next_point;
}WPS;

typedef struct WGPS //way point structure
{
  unsigned short id;  //number
  double lat;    //degree
  double lon;   //degree
  int alt;       //meters
  int spd;          //km/h
  int crs;         //degree
  int dur;       //seconds
  unsigned char npt;  //N - next, or number
}WGPS;

/*struct WGPS gway= { id, lat, log, alt, spd, crs, dur, npt };*/

typedef struct TEST
{
  int  p1;
  int  p2;
  int  p3;
  int  p4;
}TEST;

struct FSSi in={0,0,0,0,0};
struct FSSo out={128,128,128,0};
struct FSSp keep={0,0,3};
//struct FSSc ctrl={
//0.05, //Kst_roll
//0.0,  //Kast_roll
//0.05, //Kfar_roll
//0.1,  //Kst_pitch
//0.0,  //Kast_pitch
//0.1,  //Kfar_pitch
//5.0,  //Kst_speed
//0.0,  //Kast_speed
//0.02, //Kfar_speed
//0.5,  //Kst_hdg
//1.0,  //Kast_hdg
//0.5,  //Kst_alt
//1.0   //Kast_alt
//};

struct FSSc ctrl={
/*P 0.1, 0.0, 0.0, 0.2, 0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 0.0 real 2012*/
/*P 0.1, 0.0, 0.1, 0.2, 0.0, 0.2, 2.0, 0.0, 0.5, 0.0, 2.0, 0.0 real 2017-08-09*/
    0.08, 0.0, 0.08, 0.2, 0.0, 0.2, 5.0, 1.0, 0.02, 0.2, 0.0, 2.0, 0.0 
/*
0.3, //Kst_roll
0.3,  //Kast_roll
1.00, //Kfar_roll
0.3,  //Kst_pitch
0.3,  //Kast_pitch
0.1,  //Kfar_pitch
5.0,  //Kst_speed
0.0,  //Kast_speed
0.02, //Kfar_speed
0.5,  //Kst_hdg
0.0,  //Kast_hdg
0.5,  //Kst_alt
0.0   //Kast_alt
*/};
int ReturnSpeed = 30, ReturnAlt = 40;
/* waypoint, altitude, heading, speed, time, nextpoint */
//struct WPS way={0, 100, 0, 80, 10, 1};
struct WGPS gway= {  0,   0,   0,   0,   0,   0,   0,   0 };
struct TEST tst={0, 0, 0, 0};
struct FSSo servo={128,128,128,128};
/* GPS sructures */
struct GPS_DATA gps={'V',0,0,0,0,0,0};
struct CURRENT_TIME ctime={2016,9,18,17,30,0};
char g_nd=0, g_flg=0;


int roll_sum, roll_dim[FSS_SAMPLES+1], fss_j=0, ri=0, roll_pr=0,
    /*roll_var_dim[FSS_SAMPLES+1],*/  roll_long_sum=0,
    keep_roll_sum = 0;
int pitch_sum, pitch_dim[FSS_SAMPLES+1],
    /*pitch_var_dim[FSS_SAMPLES+1],*/  pitch_long_sum=0,
    keep_pitch_sum = 0, pitch_pr=0;

int speed_sum, speed_pr=0;

#ifdef MMC_CARD
int ReadParam( tFILE * f, FSSc * st);  
//int GetNextPoint( tFILE * f, WPS * wp);
int GetGPSNextPoint( tFILE * f, WGPS * wp);
#endif

int alt_error=0, alt_current, alt_var=0, alt_old;
int head_error=0,  head_current, head_var=0, head_old;
//int speed_error=0,  speed_current;

int apilot_enable = 0;

void FlStSys(FSSi * in, FSSo *out, FSSp * keep, FSSc * ctrl, TEST *tst);

void HiPriorityFunc(void);
//struct _AT91S_HID HID;
void T0isr(void); 
//void T1isr(void); 
void T2isr(void); 
void PWMCisr(void);
void SYSisr(void);
void ADCisr(void);
void ToggleTp( void );
//void ErrorPrint(void);

static int xdim[32];
static int ydim[32];
static int tdim[10], signal_sens;
static int wdim[10];
static int odim[10];
static int dalt[ALT_SAMPLES], salt = 0;
static int dspeed[SPEED_SAMPLES], sspeed = 0;
static int dgyro[GYRO_SAMPLES], sgyro = 0;
static int ti,wi,ialt=0, ispeed=0, igyro =0;
static int flag;
static int xvalue,  xsum, xi;
static int yvalue,  ysum, yi;
static int minutes, seconds, hours;
static int calibrate_compass=0;
unsigned int msb = 0;
unsigned int lsb = 0;
static int malt;
static int mspeed;
static int mgyro, vgyro;
static int v_akk;
static float teta, fi;
static int ateta;
static int ttc0 = 0, ttc2 = 0;

/********************navigaion parameters ***************************************/
double lon_from=0, lat_from=0, lat_to, lon_to, lat_start, lon_start;
double ReturnLat, ReturnLon;
float dlat, dlon, ptp_dist, distance=0, offset = 0, distance_old=0, Azimut=0, TrueAzimut=0;
int  point_number = 0, init_start_point = 0, get_new_point = 1;
int ReturnPointSet = 0, speed_var;
float  gnd_speed, speed_trim;
/********************************************************************************/



int valt_0=0, vspeed_0=0, vgyro_0=0, calibrate_sensors=1, takeoff;
char mmc_present = 0;
char waypoint_present = 0;
short int indcnt=0, indicator=4, in_signal=0;
char prntbuf[300];
char prntbuf2[100];
char prntbuf3[100];
char errbuf[50];
// external buffer which is use to read/write in MMC card
#ifdef MMC_CARD
extern char mmc_buffer[512];
#endif

unsigned int ch1 = 0;
unsigned int ch2 = 0;
unsigned int cmd_rt = 0;
unsigned int cmdcnt = 0;
unsigned int speed_var_cnt = 0;

#ifdef MMC_CARD
  tFILE *way_point = NULL, *target = NULL;
  char file_name[12];
#endif

//*--------------------------------------------------------------------------------------
//* Function Name       : main
//* Object              :
//*--------------------------------------------------------------------------------------
int main ( void )
{
  int i, k, tc=0, htmp, mheading;
  unsigned int sec = 0;// tc0sr, tc2sr;
  int dsec, calibrate_cnt=0;
  float Ymc, Xmc, Zmc, Ymr, Xmr, heading_r; 
  flag = 1;
  xvalue = 0;
  xi = 0;
  xsum = 0;
  yvalue = 0;
  yi = 0;
  ysum = 0;
  ti=0;
  takeoff = 5;
  speed_trim = 0;
  for(i=0; i<=(NUMCHAN-1);i++)
  {
    wdim[i] = WMED;
    odim[i] = WMED;
  }
  odim[2] = WMIN;
  wdim[2] = WMIN;
  wdim[NUMCHAN] = WPAUSE;
  odim[NUMCHAN] = WPAUSE;

        
  *AT91C_PIOA_PER  = AT91C_PIO_PA7 | AT91C_PIO_PA8 | AT91C_PIO_PA19;  //Enable PA8
  *AT91C_PIOA_OER  = AT91C_PIO_PA7 | AT91C_PIO_PA8 | AT91C_PIO_PA19;  //Configure in Output
  *AT91C_PIOA_SODR = AT91C_PIO_PA7 | AT91C_PIO_PA8 | AT91C_PIO_PA19;  //set reg to 1
  
#ifdef FLYSKY
  //Enable PA25,PA28,PA29,PA30
  *AT91C_PIOA_ODR =  AT91C_PIO_PA25 | AT91C_PIO_PA28 | AT91C_PIO_PA29 | AT91C_PIO_PA30;  //disable output regs
  *AT91C_PIOA_PER =  AT91C_PIO_PA25 | AT91C_PIO_PA28 | AT91C_PIO_PA29 | AT91C_PIO_PA30;  //disable output regs0x72000;

#endif  
      
  AT91F_TC0_CfgPIO();
  AT91F_TC1_CfgPIO();
  AT91F_TC2_CfgPIO();
                          
  *AT91C_TC0_IDR = 0xFFFFFFFF;
  *AT91C_TC1_IDR = 0xFFFFFFFF;
  *AT91C_TC2_IDR = 0xFFFFFFFF;
    
  *AT91C_PMC_SCER = 0x00000081;   // enable clocks
//  *AT91C_PMC_PCER = 0x80007407;   // enable clocks for periferials
//  *AT91C_PMC_PCER = 0x40007407;   // enable clocks for periferials
  *AT91C_PMC_PCER = 0xC00076F7;   // enable clocks for periferials

/*    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_FIQ);
    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_SYS);
    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_PIOA);
    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_PWMC);
    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_TC0);
    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_TC1);
    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_TC2);
    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_IRQ0);
    AT91F_PMC_EnablePeriphClock (AT91C_BASE_PMC, AT91C_ID_IRQ1);
  */
    
    //set int ptiority          
    AT91C_AIC_SMR[1]  =  1 | AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE;//AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED;
    AT91C_AIC_SMR[4]  =  1 | AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL;//AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE;
    AT91C_AIC_SMR[9]  =  1 | AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE;//AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED; 
    AT91C_AIC_SMR[10]  = 2 | AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL;//AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE;
    AT91C_AIC_SMR[12]  = 2 | AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE;//AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED;
    AT91C_AIC_SMR[14]  = 2 | AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE;//AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED;
    AT91C_AIC_SMR[31]  = 1 | AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE;//AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED;
//2222    AT91C_AIC_SMR[30]  = 1 | AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE;//AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED;

    // Vectors initialisation;
    AT91C_AIC_SVR[1]   = (unsigned)SYSisr;       //set int vector for RTT (or SYS)  
    AT91C_AIC_SVR[4]   = (unsigned)ADCisr;       //set int vector for ADC  
    AT91C_AIC_SVR[9]   = (unsigned)TWIisr;       //set int vector for TWI
    AT91C_AIC_SVR[10]  = (unsigned)PWMCisr;      //set int vector for PWMC  
    AT91C_AIC_SVR[12]  = (unsigned)T0isr;        //set int vector for TC0  
//    AT91C_AIC_SVR[13]  = (unsigned)T1isr;        //set int vector for TC1 
    AT91C_AIC_SVR[14]  = (unsigned)T2isr;        //set int vector for TC2  
    AT91C_AIC_SVR[31]  = (unsigned)HiPriorityFunc;  //set int vector for AIC IRQ1  
//2222    AT91C_AIC_SVR[30]  = (unsigned)HiPriorityFunc;  //set int vector for AIC IRQ0  
                              
    *AT91C_AIC_IECR = 0x80005412;   // enable interrupts from TC0, TC2, PWMC, IRQ1,
//2222    *AT91C_AIC_IECR = 0x40005412;   // enable interrupts from TC0, TC2, PWMC, IRQ0,
                                    // ADC, IRQ1- as software interrupt;      
    *AT91C_AIC_DCR = 0x1;     // protected mode enable 
    *AT91C_TCB_BMR = AT91C_TCB_TC0XC0S_NONE; // no external sourses fo timer
    *AT91C_TCB_BMR = AT91C_TCB_TC1XC1S_NONE; // no external sourses fo timer
    *AT91C_TCB_BMR = AT91C_TCB_TC2XC2S_NONE; // no external sourses fo timer
    
    *AT91C_TCB_BCR = 0x1;   // synchronization 
    
//    *AT91C_TC0_CMR = AT91C_TC_CLKS_TIMER_DIV2_CLOCK |                //  config for capture TC0
//    AT91C_TC_LDBSTOP | AT91C_TC_ETRGEDG_RISING | AT91C_TC_ABETRG | 
//    AT91C_TC_LDRA_FALLING | AT91C_TC_LDRB_RISING;  
//    *AT91C_TC0_IER = AT91C_TC_LDRBS;

    *AT91C_TC0_CMR = AT91C_TC_CLKS_TIMER_DIV2_CLOCK |                //  config for capture TC0
    AT91C_TC_ETRGEDG_RISING | AT91C_TC_ABETRG | 
    AT91C_TC_LDRA_FALLING;  
    *AT91C_TC0_IER = AT91C_TC_LDRAS;

//    *AT91C_TC1_CMR = AT91C_TC_CLKS_TIMER_DIV2_CLOCK |                //  config for capture TC1
//    AT91C_TC_LDBSTOP | AT91C_TC_ETRGEDG_RISING | AT91C_TC_ABETRG | 
//    AT91C_TC_LDRA_FALLING | AT91C_TC_LDRB_RISING;  

    *AT91C_TC1_CMR = AT91C_TC_CLKS_TIMER_DIV2_CLOCK |                //  config for capture TC1
    AT91C_TC_ETRGEDG_RISING | AT91C_TC_ABETRG | 
    AT91C_TC_LDRA_FALLING;  


#ifdef FLYSKY     

    //for new flysky receiver
    *AT91C_TC2_CMR = AT91C_TC_CLKS_TIMER_DIV2_CLOCK |                //  config for capture TC2
    AT91C_TC_ETRGEDG_RISING  | //AT91C_TC_ABETRG | 
    AT91C_TC_CPCTRG;// AT91C_TC_LDRB_FALLING |
    //AT91C_TC_LDBSTOP | AT91C_TC_LDRA_RISING; 
    *AT91C_TC2_IER = AT91C_TC_CPCS;   
    *AT91C_TC2_RC = 60;  
    
#else    

    *AT91C_TC2_CMR = AT91C_TC_CLKS_TIMER_DIV3_CLOCK |                //  config for capture TC2
    AT91C_TC_ETRGEDG_FALLING | AT91C_TC_ABETRG | 
    AT91C_TC_LDRA_FALLING | AT91C_TC_CPCTRG;
    *AT91C_TC2_RC = 0x7fff;  
    *AT91C_TC2_IER = AT91C_TC_LDRAS | AT91C_TC_CPCS;
    //*AT91C_TC2_IER = AT91C_TC_ETRGS;   //???


#endif    

//    *AT91C_TC2_CMR = AT91C_TC_CLKS_TIMER_DIV3_CLOCK |            // A to B capture TC2
//    AT91C_TC_ETRGEDG_RISING | AT91C_TC_ABETRG | 
//    AT91C_TC_LDRA_FALLING | AT91C_TC_LDRB_RISING;  
//    *AT91C_TC2_IER = AT91C_TC_ETRGS;   //???
    
    *AT91C_TC0_CCR = AT91C_TC_CLKEN;   //enable clk  
    *AT91C_TC1_CCR = AT91C_TC_CLKEN;   //enable clk  
    *AT91C_TC2_CCR = AT91C_TC_CLKEN;   //enable clk  

    *AT91C_TC0_CCR = AT91C_TC_SWTRG;   //start clock?
    *AT91C_TC1_CCR = AT91C_TC_SWTRG;   //start clock?
    *AT91C_TC2_CCR = AT91C_TC_SWTRG;   //start clock?
    
    //   - 

    *AT91C_PWMC_DIS = 0xF;
    *AT91C_PIOA_PDR = 0x800;
    *AT91C_PIOA_ASR = 0;
    *AT91C_PIOA_BSR = 0x800;
    *AT91C_PWMC_MR=0x00000000;     //  AT91C_PWMC_DIVA | AT91C_PWMC_PREA |
                                  //  AT91C_PWMC_DIVB | AT91C_PWMC_PREB; 
    *AT91C_PWMC_CH0_CMR=5;//*AT91C_CH0_CMR=5;  
    *AT91C_PWMC_CH0_CPRDR = WIDE;
    *AT91C_PWMC_CH0_CDTYR = WDUTY; 
    *AT91C_PWMC_CH0_CMR=5 | AT91C_PWMC_CPD;  
    *AT91C_PWMC_IER=AT91C_PWMC_CHID0;
    *AT91C_PWMC_ENA=AT91C_PWMC_CHID0;
//    *AT91C_AIC_EOICR = 0;    

    InitUSART0();
    InitUSART1();
    GPS_Init();

    // Configure the RTT:
    *AT91C_RTTC_RTMR = 0x0200 | AT91C_RTTC_RTTINCIEN |
    AT91C_RTTC_RTTRST;
    //    *AT91C_RTTC_RTAR = 0x00000010;
    //TWI_Initialization
    TwiInit();
    
    //ADC Initialization
    
    *AT91C_ADC_MR =(8 << 24) | (7 << 16) | (11<<8);// | AT91C_ADC_TRGSEL_TIOA0 | AT91C_ADC_TRGEN_EN; 
    *AT91C_ADC_CHER = AT91C_ADC_CH3 | AT91C_ADC_CH4 | AT91C_ADC_CH5| AT91C_ADC_CH6;
    *AT91C_ADC_IER = AT91C_ADC_CH3 | AT91C_ADC_CH4 | AT91C_ADC_CH5| AT91C_ADC_CH6;

  //Calibration compass switch - PA7
   *AT91C_PIOA_ODR = 0x80;  //Configure in Input
   *AT91C_PIOA_PER = 0x80;  //Enable PA7
  //Calibration compass switch - PA07
   TPOFF
    
  if((*AT91C_PIOA_PDSR & 0x80)==0)
  {
    calibrate_compass=1;
    *AT91C_WDTC_WDMR=0x0008000; //disable watch dog
    indicator = 8;
  }
//  else
//  {
   *AT91C_PIOA_OER = 0x80;  
/* if no calibrate configure as trest point */
//  } 

  if(calibrate_sensors == 0) 
  {
    sprintf(errbuf, "\r\nERROR! System restart...\r\n");
    //write_str_pdc_USART0((unsigned char*)errbuf);
    goto main_cycle;
  }
  
while(*AT91C_RTTC_RTVR <= 20*64);    //  make comment for fast start at debugging
//calibrate_sensors = 0;               //  do not comment for fast start at debugging

 
sprintf(prntbuf, VERSION);
write_str_USART0((unsigned char*)prntbuf);  

/*********************************WD_ENABLING*************************/
#ifdef WATCH_DOG
*AT91C_WDTC_WDMR=0x04002400; //for 4 seconds
#else
*AT91C_WDTC_WDMR=0x0008000;
#endif

/***********************TUNE SYSTEM TO FIRST START*******************/

    sprintf(prntbuf, "Waiting GPS..\r\n");
    write_str_USART0((unsigned char*)prntbuf);  

#ifdef WAIT_GPS
  while(1)
  {
    if( g_nd == 1 )
    {
      sprintf(prntbuf3,"%02d:%02d:%02d GPS=%c%d LAT=%+2.8f LON=%+3.8f HDG=%3.1f SPD=%3.1f ALT=%3.1f\r\n",
      ctime.hour, ctime.minute, ctime.secunde, gps.status, gps.SatCount, gps.Latitude, gps.Longitude,
      gps.Heading, gps.Speed, gps.Altitude);
      write_str_pdc_USART0((unsigned char*)prntbuf3);
      g_nd = 0;
      if (gps.status == 'A') break;
    }
  }
#endif

    sprintf(prntbuf, "GPS ready..\r\n");
    write_str_USART0((unsigned char*)prntbuf);  


#ifdef MMC_CARD
  
    //mmc_initialisation
  /**** System init ****/
  Init_CP_WP();

  //chek for CP and WP
  //CP - card present
  if((((m_pPio->PIO_PDSR) & BIT10)==0) && (((m_pPio->PIO_PDSR) & BIT9)==0))
  { 

    if (initMMC() == MMC_SUCCESS) // card found
    {
  
      if(gps.status == 'A')  TPON
      else GPEON
      //card_state |= 1;
      memset(&mmc_buffer,0,512);
      mmcReadRegister (10, 16);
      mmc_buffer[7]=0;
      k = fatInit();
      if( k >= 0 )
      { 
        waypoint_present = 0;     
        //check for waipoint.txt exists;
/*        if((way_point = fileOpen("WAYPOINTTXT", 'a')) == NULL )
        {
          sprintf(prntbuf, "TRACK file waypoint.txt not found %d\n\r", k);
          write_str_USART0((unsigned char*)prntbuf);
        }        
        else waypoint_present = 1;*/
        
        if((way_point = fileOpen("GPSPOINTTXT", 'a')) == NULL )
        {
          sprintf(prntbuf, "TRACK file gpspoint.txt not found %d\n\r", k);
          write_str_USART0((unsigned char*)prntbuf);
        }        
        else waypoint_present = 2;
        if( waypoint_present != 0)
        {
          sprintf(prntbuf, "Init from MMC card..Hidden sectors = %d\n\r", k);
          write_str_pdc_USART0((unsigned char*)prntbuf);
          ReadParam( way_point, &ctrl);
//         if( waypoint_present == 1) GetNextPoint( way_point, &way);
         if( waypoint_present == 2)
         {
           //calculate  a table course and time 
          point_number = 0;
          if(GetGPSNextPoint( way_point, &gway) == 0)
          {
            lat_from = gway.lat*1000000;
            lon_from = gway.lon*1000000;
          } 
 
          sprintf(prntbuf,"G%d,%2.6f,%2.6f,%d,%d,%d,%d,%d\r\n",point_number,gway.lat,gway.lon,gway.alt,gway.spd,gway.crs,gway.dur,gway.npt);
          write_str_USART0((unsigned char*)prntbuf);

          point_number++;
          while(1)
          {
             if(GetGPSNextPoint( way_point, &gway) == 0)
             {
               lat_to = gway.lat*1000000;
               lon_to = gway.lon*1000000;
             } 
             else
             {
               break;
             }
              dlat = (float)(lat_to - lat_from);
              dlon = (float)(lon_to - lon_from)*cos((lat_from*PI)/18E7);
              distance = (sqrt(dlat*dlat + dlon*dlon))/9;
  
  
              if (dlat >= 0 && dlon >= 0)
                      if(dlat==0) Azimut = PI/2;
                      else Azimut = atan(dlon/dlat);
              else if (dlat <= 0 && dlon >= 0)
                      if(dlat==0) Azimut = PI/2;
                      else Azimut = PI+atan(dlon/dlat);
              else if (dlat < 0 && dlon < 0)
                      if(dlat==0) Azimut = 3*PI/2;
                      else Azimut = PI+atan(dlon/dlat);
              else
                      if(dlat==0) Azimut = 3*PI/2;
                      else Azimut = (2*PI)+atan(dlon/dlat);
  

              gway.crs = ((Azimut*180)/PI);
              if(gway.spd > 0)
              {
               gway.dur = 3.6 * distance/gway.spd;
//              ttp_hr  =time_to_point/3600;
//              ttp_min =(time_to_point/60)%60;
//              ttp_sec =time_to_point-(ttp_hr*3600)-(ttp_min*60);
              }
              lat_from = gway.lat*1000000;
              lon_from = gway.lon*1000000;
              
              sprintf(prntbuf,"G%d,%2.6f,%2.6f,%dM,%dkM/h,%dD,%ds,%d,%dM\r\n",point_number,gway.lat,gway.lon,gway.alt,gway.spd,gway.crs,gway.dur,gway.npt,(int)distance);
              write_str_USART0((unsigned char*)prntbuf);
              point_number++;

          }
         }
        }
        for(i=0; i<999; i++)
        {
          sprintf(file_name, "TRACK%03dLOG", i);
          if((target=fileOpen(file_name,'c'))!=NULL) break; 
        }
        if(target!=NULL)
        {
          mmc_present = 1;
          fileWrite(target, VERSION, strlen(VERSION));
        }
      }
      else
      {
        sprintf(prntbuf, "SD card error..Hidden sectors = %d\n\r", k);
        write_str_USART0((unsigned char*)prntbuf);
      }
    }
  }
  else
  {
    
    sprintf(prntbuf, "MMC card not found!\n\r");
    write_str_pdc_USART0((unsigned char*)prntbuf);  
  }
#endif  

/*    sprintf(prntbuf, "Kst_roll:%d Kast_roll:%d Kfar_roll:%d\r\n Kst_pitch:%d Kast_pitch:%d Kfar_pitch:%d\r\n\
Kst_speed:%d Kast_speed:%d Kst_hdg:%d Kast_hdg:%d Kst_alt:%d Kast_alt:%d\r\n",
            (int)(ctrl.Kst_roll*10), (int)(ctrl.Kast_roll*10), (int)(ctrl.Kfar_roll*10), (int)(ctrl.Kst_pitch*10),
            (int)(ctrl.Kast_pitch*10), (int)(ctrl.Kfar_pitch*10),(int)(ctrl.Kst_speed*10), (int)(ctrl.Kast_speed*10),
            (int)(ctrl.Kst_hdg*10), (int)(ctrl.Kast_hdg*10), (int)(ctrl.Kst_alt*10), (int)(ctrl.Kast_alt*10));
    sprintf(prntbuf, "%f %f %f %f %f %f %f %f %f %f %f %f\r\n",
            ctrl.Kst_roll,ctrl.Kast_roll,ctrl.Kfar_roll,ctrl.Kst_pitch,ctrl.Kast_pitch,ctrl.Kfar_pitch,
            ctrl.Kst_speed,ctrl.Kast_speed,ctrl.Kst_hdg,ctrl.Kast_hdg,ctrl.Kst_alt,ctrl.Kast_alt);
    write_str_USART0((unsigned char*)prntbuf);  */

if(calibrate_compass == 1)
{
    indicator = 8;
    while(*AT91C_RTTC_RTVR <= 20*64);
    if(gps.status == 'A')  TPON
    else GPEON
//  User Calibration    
    TwiWrite(0x21,0x0, 0x0,0x43);
    sprintf(prntbuf, "Please calibrate compass..\r\n");
    write_str_pdc_USART0((unsigned char*)prntbuf);  
    
    while(*AT91C_RTTC_RTVR <= 60*64);
    
    TwiWrite(0x21,0x0,0x0,0x45);
    sprintf(prntbuf, "Compass calibrate complete..\r\n");
    write_str_pdc_USART0((unsigned char*)prntbuf);  
    indicator = 32;
    TPOFF
    GPEOFF
    while(*AT91C_RTTC_RTVR <= 64*64);
}
else
{
     sprintf(prntbuf, "Compass calibrate skip..\r\n");
     write_str_USART0((unsigned char*)prntbuf);  
}

    sprintf(prntbuf, "Sensors calibrate..\r\n");
    write_str_USART0((unsigned char*)prntbuf);  
while(calibrate_sensors == 1)
{
  indicator = 32;
  if( cmd_rt == 1 )
  {
      valt_0 = valt_0+malt;
      vspeed_0 =vspeed_0+mspeed;        
      vgyro_0 = vgyro_0+mgyro;     
      cmd_rt = 0;  
      if( calibrate_cnt == 255 )
      {
        valt_0 = valt_0>>8;
        vspeed_0 = vspeed_0>>8; 
        vgyro_0 =vgyro_0>>8;     
        calibrate_sensors = 0;
        calibrate_cnt = 0;
        sprintf(prntbuf, "Sensors calibrate complete..\r\n");
        write_str_USART0((unsigned char*)prntbuf);  
        break;
      }
      else
      { 
        calibrate_cnt++;  
      }
  }   
}        
/****************************MAIN CYCLE*******************************/

main_cycle:    

k=0;
tc=0;

while(calibrate_sensors == 0)
{
   if( cmd_rt==1 )
   {
     
         
/************************  GPS PROCESSING ****************************/         
/*      if (g_nd == 1 ) 
      {
        g_nd = 0;
        i = GPSProcessing();              
        if(i !=0 )
        {
          sprintf( errbuf, "\r\nGPS Error %d\r\n", i );
          write_str_pdc_USART0((unsigned char*)errbuf);
        }
      }*/
/************************  GPS PROCESSING ****************************/         

        //TPON
        tc++; 
        if(tc==64) tc=0; 
     
        switch(k++)
        {

          case 0:     TwiWrite(0x21,0x474E,0x2,0x00);break;
          case 1:     Xmc = TwiRead(0x21,0x41,0x1);break;
          case 2:     TwiWrite(0x21,0x474E,0x2,0x03);break;
          case 3:     Ymc = TwiRead(0x21,0x41,0x1); break;
          case 4:     TwiWrite(0x21,0x474E,0x2,0x04);break;
          case 5:     mheading = (TwiRead(0x21,0x41,0x1)); break; //heading*10
          case 6:  
            
#ifdef COMPENSATION              

              Zmc=-360;
              Xmr=Xmc*cos(fi) + Ymc*sin(teta)*sin(fi)-Zmc*cos(teta)*sin(fi);
              Ymr=Ymc*cos(teta)+Zmc*sin(teta);

              if((Xmr >= 0) && (Ymr < 0))
              {
                heading_r = -atan(Ymr/Xmr);    
              }
              else if((Xmr < 0) && (Ymr < 0))
              {
                heading_r = PI-atan(Ymr/Xmr);    
              }
              else if((Xmr < 0) && (Ymr >= 0))
              {
                heading_r = PI-atan(Ymr/Xmr);    
              }
              else 
              {
                heading_r = (2*PI)-atan(Ymr/Xmr);    
              }
//              heading_r= (heading_r*180)/PI;
              heading_r= (heading_r*1800)/PI;   //heading*10
              
//              if(gps.status == 'A') in.heading = gps.Heading*10;
//              else in.heading = (int)heading_r;  
              
              if((gps.Speed >= GPS_MIN_SPEED) && (gps.status == 'A')) in.heading = gps.Heading*10;
              else in.heading = (int)heading_r;              
              
#else

//              if(gps.status == 'A') in.heading = gps.Heading;
//              else in.heading = mheading;              

              if((gps.Speed >= GPS_MIN_SPEED) && (gps.status == 'A')) in.heading = gps.Heading*10;
              else in.heading = mheading;              
              
#endif              
              break; 
          case 7:  

            alt_current = in.altitude;
            
            if((alt_current >= 10) && (takeoff > 0))
            {
              takeoff--;
            }
#ifdef  ONLY_THROTTLE           

            if( takeoff > 0 )  //take off mode
            {
              apilot_enable = 1;              
              indicator = 64;
            }              
            else 
            {
              apilot_enable = 2;        //fligt mode        
              indicator = 64;
            }

#else
            if(in_signal == 1)
            {
             apilot_enable = 0;          //hand control mode    
             indicator = 128;
            }
            else if( takeoff > 0 )  //take off mode
            {
              apilot_enable = 1;              
              indicator = 64;
            }              
            else 
            {
              apilot_enable = 2;        //fligt mode        
              indicator = 64;
            }
#endif            
/*            
           if(in_signal == 1)
          {
            if(wdim[4] > (WMED+WMAX)/2)
            { 
              apilot_enable = 0;
              indicator = 128;
            }
            else if( wdim[4] < (WMED+WMIN)/2)
            {
              apilot_enable = 2;              
              indicator = 64;
            }
            else 
            {
//              apilot_enable = 0;              
//              indicator = 128;
              apilot_enable = 1;              
              indicator = 64;
            }
          }
            
          if(apilot_enable == 1)
          {
            keep.roll = 0;
            keep.pitch = 0;
            keep.speed = 80;
          }
*/
            

          if(apilot_enable != 0)
          {
            head_current = in.heading;
//            alt_current = in.altitude;
//            speed_current = in.speed;
            alt_var = alt_current-alt_old;
            htmp = head_current-head_old;
//          speed_var = speed_current - speed_old;
            alt_old=in.altitude;
            head_old=in.heading;
//          speed_old =in.speed;
/**************************************heading control****************************************************/         
         
            htmp =(gway.crs*10) - head_current ;
            if(htmp >= 1800) head_error =  htmp-3600;
            else if(htmp < -1800) head_error = htmp + 3600 ;
            else head_error = htmp;

/********************************** Variant 2 with  averaging ********************************************/           
//
//            keep_roll_sum = keep_roll_sum + head_error*ctrl.Kst_hdg - head_var*ctrl.Kast_hdg - keep.roll;
//            
//            if (keep_roll_sum >= 80*AP2_SAMPLES)  keep_roll_sum=80*AP2_SAMPLES;
//            else if (keep_roll_sum <= -80*AP2_SAMPLES)  keep_roll_sum=-80*AP2_SAMPLES;
//
//            keep.roll =  keep_roll_sum/AP2_SAMPLES;
//            
/********************************** Variant 1 ************************************************************/           
            if(apilot_enable == 2)
            {
              keep.roll = (head_error*ctrl.Kst_hdg)  - (head_var*ctrl.Kast_hdg);                         
              if (keep.roll >= 150)  keep.roll=150;                                                          
              else if (keep.roll <= -150)  keep.roll=-150;
            }
            else
            {
              keep.roll = 0;
            }
            
/*********************************************************************************************************/           

/**************************************altituge control****************************************************/         

            alt_error =  gway.alt - alt_current;

/********************************** Variant 2 with  averaging ********************************************/           
//            keep_pitch_sum = keep_pitch_sum - alt_var*ctrl.Kast_alt + alt_error*ctrl.Kst_alt - keep.pitch;
//
//            if (keep_pitch_sum >= 320)  keep_pitch_sum=320;
//            else if (keep_pitch_sum <= -320)  keep_pitch_sum=-320;
//
//            keep.pitch =  keep_pitch_sum/4;
//
/********************************** Variant 1 ************************************************************/           
          keep.pitch = (alt_error*ctrl.Kst_alt)  - (alt_var*ctrl.Kast_alt);                            
          if (keep.pitch >= 100)  keep.pitch=100;                                                        
          else if (keep.pitch <= -100)  keep.pitch=-100;                                                  
/*********************************************************************************************************/           

/************************************* speed control ******************************************************/         
//            speed_error =  way.speed_requested - speed_current;
            keep.speed = gway.spd;
          }
          else
          {
              keep.roll = 0;
              keep.pitch = 0;
#ifdef DRAGON_SPORTSTER      
              keep.speed = gway.spd;
#elif  DUSTY  1
              keep.speed = gway.spd;
#else        
              keep.speed = gway.spd;
#endif        
           
          }
          break;
          default: k=0;break;
   }

               
                
/**************************************  NAVIGATION **********************************************************/    
        
     if(tc==10)
     {
      // if(apilot_enable != 0)
        if(waypoint_present !=0 )
       {
         if( init_start_point == 0 )
         {
            point_number = 0;
            fileReset(way_point);
            if(GetGPSNextPoint( way_point, &gway) == 0)
            {
              lat_to = lat_start = gway.lat*1000000;
              lon_to = lon_start = gway.lon*1000000;
            } 
            else waypoint_present = 0;
            init_start_point = 1;
         }
         if( get_new_point == 1 ) 
         {
            get_new_point = 0;
            lat_from = lat_to;
            lon_from = lon_to;
            point_number++;
            
            if(GetGPSNextPoint( way_point, &gway) == 0)
            {
              lat_to = gway.lat*1000000;
              lon_to = gway.lon*1000000;
            } 
            else
            {
              waypoint_present = 0;
            }
            /*Here we calulate TrueAzimut; it is constant from point A to point B. 
              It is heading wich we need have at aproaching to the point B 
              This operation we make one time at changing to next point*/
            dlat = (float)(lat_to - lat_from);
            dlon = (float)(lon_to - lon_from)*cos(((float)lat_from*PI)/18E7);
            ptp_dist = (sqrt(dlat*dlat + dlon*dlon))/9;

            if (dlat >= 0 && dlon >= 0)
                    if(dlat==0) TrueAzimut = PI/2;
                    else TrueAzimut = atan(dlon/dlat);
            else if (dlat <= 0 && dlon >= 0)
                    if(dlat==0) TrueAzimut = PI/2;
                    else TrueAzimut = PI+atan(dlon/dlat);
            else if (dlat < 0 && dlon < 0)
                    if(dlat==0) TrueAzimut = 3*PI/2;
                    else TrueAzimut = PI+atan(dlon/dlat);
            else
                    if(dlat==0) TrueAzimut = 3*PI/2;
                    else TrueAzimut = (2*PI)+atan(dlon/dlat);

          }
            
         dlat = (float)(lat_to - gps.Latitude*1000000);
         dlon = (float)(lon_to - gps.Longitude*1000000)*cos((gps.Latitude*PI)/18E1);
         distance = (sqrt(dlat*dlat + dlon*dlon))/9;
         if (dlat >= 0 && dlon >= 0)
                  if(dlat==0) Azimut = PI/2;
                  else Azimut = atan(dlon/dlat);
         else if (dlat <= 0 && dlon >= 0)
                  if(dlat==0) Azimut = PI/2;
                  else Azimut = PI+atan(dlon/dlat);
         else if (dlat < 0 && dlon < 0)
                  if(dlat==0) Azimut = 3*PI/2;
                  else Azimut = PI+atan(dlon/dlat);
         else
                  if(dlat==0) Azimut = 3*PI/2;
                  else Azimut = (2*PI)+atan(dlon/dlat);
  
         offset = distance*sin( TrueAzimut - Azimut);

         if(gnd_speed > 0)
         {
          gway.dur = 3.6 * distance/gway.spd;
//         ttp_hr  =time_to_point/3600;
//         ttp_min =(time_to_point/60)%60;
//         ttp_sec =time_to_point-(ttp_hr*3600)-(ttp_min*60);
         }
         else  gway.dur = 8888;
 

          
         if((distance >= 2*RadRazv)&&(offset!=0)&&
         (abs(Azimut - TrueAzimut)<(PI/2))&&(distance < ptp_dist))
         { 
           if(offset >=0)
           {
    //              htmp =(int)((((Azimut-((Sign(offset))*acos(offset/distance)-atan(100/offset)))*180)/PI)*10);
    //              htmp =(int)((((Azimut-(PI/2-atan((200*200)/(20*offset*abs(offset)))))*180)/PI)*10);
                    htmp =(int)((TrueAzimut-(PI/2-atan(RadRazv/offset)))*(180/PI)*10);
    //              htmp =(int)((((Azimut-(asin(offset/distance)))*180)/PI)*10);
           }
           else
           {
                    htmp =(int)((TrueAzimut+(PI/2+atan(RadRazv/offset)))*(180/PI)*10);
           }
         }
         else
         {
            htmp =(int)(((Azimut*180)/PI)*10);
         }
         if(distance < 2*RadRazv)
         {
          if((distance_old < distance)||(distance < RadRazv))
          {
            if(gway.npt != 0) get_new_point = 1;
            htmp =(int)(((TrueAzimut*180)/PI)*10);
          }
         }
         distance_old = distance;
         if(htmp > 3599)
           gway.crs = (htmp - 3600)/10;
         else if(htmp < 0)
           gway.crs = (htmp + 3600)/10;
         else gway.crs = htmp/10;
       }
       else
       {

         dlat = (float)(ReturnLat - gps.Latitude*1000000);
         dlon = (float)(ReturnLon - gps.Longitude*1000000)*cos((gps.Latitude*PI)/18E1);
         distance = (sqrt(dlat*dlat + dlon*dlon))/9;
         if (dlat >= 0 && dlon >= 0)
                  if(dlat==0) Azimut = PI/2;
                  else Azimut = atan(dlon/dlat);
         else if (dlat <= 0 && dlon >= 0)
                  if(dlat==0) Azimut = PI/2;
                  else Azimut = PI+atan(dlon/dlat);
         else if (dlat < 0 && dlon < 0)
                  if(dlat==0) Azimut = 3*PI/2;
                  else Azimut = PI+atan(dlon/dlat);
         else
                  if(dlat==0) Azimut = 3*PI/2;
                  else Azimut = (2*PI)+atan(dlon/dlat);
         
         htmp =(int)(((Azimut*180)/PI)*10);
         if(htmp > 3599)
         gway.crs = (htmp - 3600)/10;
         else if(htmp < 0)
         gway.crs = (htmp + 3600)/10;
         else gway.crs = htmp/10;
         gway.spd = ReturnSpeed;
         gway.alt = ReturnAlt;
         if(gnd_speed > 0)
         {
          gway.dur = 3.6 * distance/gway.spd;
//         ttp_hr  =time_to_point/3600;
//         ttp_min =(time_to_point/60)%60;
//         ttp_sec =time_to_point-(ttp_hr*3600)-(ttp_min*60);
         }
         else  gway.dur = 8888;
         gway.lat = ReturnLat/10000000;         
         gway.lon = ReturnLon/10000000;         
       }
        
     }     

/************************************* tracking control ***************************************************/         
/*       if(tc==10)
       {
#ifdef ONLY_THRUTTLE
          if(apilot_enable == 0)
#else
          if(apilot_enable == 2)
#endif            
        {

          if(way.time_requested == 0)
          {
#ifdef MMC_CARD
           if(mmc_present!=0)
           { 
            GetNextPoint( way_point, &way);
           }
#endif   
          }
          else way.time_requested--;
        }
       }


*/     
/**************************************  LOGGING **********************************************************/         
       if(tc==20)
       { 
        dsec = *AT91C_RTTC_RTVR;          ///64 
        sec=dsec>>6;                       
        hours = sec/3600;                 //
        minutes = sec/60 - (hours*60);    //  
        seconds = sec%60;                 //
        WDRESET
         
/*         
          sprintf(prntbuf, "%d %d %d %d %d %d %d %d %d %d %d %03d %03d %03d %03d %02d:%02d:%02d\r\n", 
                tdim[0],tdim[1],tdim[2],tdim[3],tdim[4],tdim[5],
                tdim[6],tdim[7],tdim[8],in.pitch, in.roll, in.heading,
                  in.altitude, in.speed, vgyro, hours, minutes, seconds);

         write_str_pdc_USART0((unsigned char*)prntbuf);
*/
        
/*      sprintf(prntbuf, "%d %d %d %d %d %d %d %d %03d %03d %03d %03d %02d:%02d:%02d AE=%d\r\n", 
                wdim[0],wdim[1],wdim[2],wdim[3],wdim[4],wdim[8],
                in.pitch, in.roll, in.heading,in.altitude, in.speed,
                vgyro, hours, minutes, seconds, apilot_enable);

         write_str_pdc_USART0((unsigned char*)prntbuf);
*/       
//          sprintf(prntbuf, "O:%d %d %d %d I:%d %d %d %d %d %d K:%d %d %d %d:%d:%d S=%d E=%d NP=%d T=%d AT=%d Vbat=%d,%d\r\n", servo.ailerons,
//                 servo.elevator, servo.rudder, servo.throttle, in.roll/10 , in.pitch/10,  in.heading/10, in.altitude,
//                 in.speed, vgyro,  keep.roll, keep.pitch, keep.speed, hours, minutes, seconds, in_signal, apilot_enable,
//                 way.point_num,way.time_requested,ateta,(v_akk*9)/1000,((v_akk*9)%1000)/10);
         
//      sprintf(prntbuf, "O:%3d %3d %3d %3d I:%3d %3d %3d %3d %3d %4d K:%4d %4d %3d %02d:%02d:%02d S=%d E=%d NP=%d T=%d AT=%d Vbat=%d,%02d\r\n", servo.ailerons,
//               servo.elevator, servo.rudder, servo.throttle, in.roll/10 , in.pitch/10,  in.heading/10, in.altitude,
//               in.speed, vgyro,  keep.roll, keep.pitch, keep.speed, ctime.hour, ctime.minute, ctime.secunde, in_signal, apilot_enable,
//               way.point_num,way.time_requested,ateta,(v_akk*9)/1000,((v_akk*9)%1000)/10);



//         sprintf(prntbuf,"%02d:%02d:%02d GPS=%c%d DST =%d HDG=%.0f THDG=%.0f RHDG=%d OFF=%d SP=%d ALT=%d WP=%d TTP=%d \r\n",
//         ctime.hour, ctime.minute, ctime.secunde, gps.status, gps.SatCount,(int)distance,((Azimut*180)/PI),
//         ((TrueAzimut*180)/PI), gway.crs/10, (int)offset, gway.spd, gway.alt, gway.id, gway.dur);

        
/*        
        strcat(prntbuf,"*********");          
        sprintf(prntbuf2, "sector =%04d cluster =%04d\r\n", calibrate_cnt/4, calibrate_cnt/32);
        strcat(prntbuf,prntbuf2);          
        calibrate_cnt++;
*/

/*        sprintf(prntbuf, "%5d %5d %5d %5d %5d %5d %5d %5d %5d %5d %5d %5d %5d %5d %5d %02d:%02d:%02d E=%d\r\n", odim[0],
                         odim[1], odim[2], odim[3], odim[4], odim[5], odim[6], tdim[7], tdim[8], in.pitch/10,
                         in.roll/10, in.heading/10, in.altitude , in.speed, vgyro/4, hours, minutes, seconds, apilot_enable);

        write_str_pdc_USART0((unsigned char*)prntbuf);
        
*/  
          
          
          if(ttc0++ >= 5) sprintf( errbuf, "\r\nT2 Error %08X\r\n", *AT91C_TC0_SR );
          if(ttc2++ >= 5) sprintf( errbuf, "\r\nT2 Error %08X\r\n", *AT91C_TC2_SR );
          
          
#ifdef EXEL_COMPATIBLE_PRINT
         prntbuf[0] = 0;
        sprintf(prntbuf, "%02d:%02d:%02d,J,%3d,%3d,%3d,%3d,I,%3d,%3d,%03d,%3d,%3d,%3d,K,%4d,%4d,%3d,%03d,%3d,P,%c%d,S%d,E%d,NP%03d,%03d,\
% 5d,% 5d,%3d,V,%d.%02d\r\n",
                ctime.hour, ctime.minute, ctime.secunde, servo.ailerons, servo.elevator, servo.rudder, servo.throttle,
                in.roll/10 , in.pitch/10,  in.heading/10, in.altitude, in.speed, (int)gnd_speed,
                keep.roll/10, keep.pitch/10, keep.speed, (int)((Azimut*180)/PI), gway.alt, gps.status, gps.SatCount,  
//                in_signal, apilot_enable, gway.id, (int)((TrueAzimut*180)/PI),(int)distance,(int)offset,(int)speed_trim,(v_akk*9)/1000,((v_akk*9)%1000)/10);
                in_signal, apilot_enable, gway.id, (int)((TrueAzimut*180)/PI),(int)tst.p1,(int)tst.p2,(int)tst.p3,(v_akk*9)/1000,((v_akk*9)%1000)/10);

        
        
//           tc0sr = *AT91C_TC0_SR; 
//           tc2sr = *AT91C_TC2_SR; 

        
/*        
          sprintf(prntbuf, "%02d:%02d:%02d,J,%3d,%3d,%3d,%3d,I,%3d,%3d,%03d,%3d,%3d,%3d,K,%4d,%4d,%3d,%03d,%3d,P,%c%d,S%d,E%d,\
V,%d.%02d,T0,%08X,T2,%08X\r\n",
                ctime.hour, ctime.minute, ctime.secunde, servo.ailerons, servo.elevator, servo.rudder, servo.throttle,
                in.roll/10 , in.pitch/10,  in.heading/10, in.altitude, in.speed, (int)gnd_speed,
                keep.roll/10, keep.pitch/10, keep.speed, (int)((Azimut*180)/PI), gway.alt, gps.status, gps.SatCount,  
                in_signal, apilot_enable,(v_akk*9)/1000,((v_akk*9)%1000)/10, tc0sr, tc2sr);
*/
        strcat(prntbuf,errbuf);   
        errbuf[0]=0;
        write_str_pdc_USART0((unsigned char*)prntbuf);
#endif  
          
          
          
          
          
#ifdef APILOT_INFO_PRINT
        prntbuf[0] = 0;
        sprintf(prntbuf, "S,%3d,%3d,%3d,%3d,D,%3d,%3d,%03d,%3d,%3d,K,%4d,%4d,%3d,%02d:%02d:%02d,%c%d,S%d,E%d,NP%03d,%03d,V=%d.%02d\r\n",
                servo.ailerons, servo.elevator, servo.rudder, servo.throttle,
                in.roll/10 , in.pitch/10,  in.heading/10, in.altitude, in.speed,
                keep.roll/10, keep.pitch/10, keep.speed, ctime.hour, ctime.minute, ctime.secunde, gps.status, gps.SatCount,  
                in_signal, apilot_enable,  gway.npt, gway.crs, (v_akk*9)/1000,((v_akk*9)%1000)/10);

        strcat(prntbuf,errbuf);   
        errbuf[0]=0;
        write_str_pdc_USART0((unsigned char*)prntbuf);
#endif  
  
          
         prntbuf2[0] = 0;
#ifdef TRACKING_INFO_PRINT
       
         sprintf(prntbuf2,"%02d:%02d:%02d GPS=%c%d DST =%d HDG=%.0f THDG=%.0f OFF=%d RHDG=%d SP=%d ALT=%d WP=%d TTP=%d \r\n",
         ctime.hour, ctime.minute, ctime.secunde, gps.status, gps.SatCount,(int)distance,((Azimut*180)/PI),
         ((TrueAzimut*180)/PI), (int)offset, in.heading/10, (int)gnd_speed, in.altitude, gway.id, gway.dur);
         write_str_pdc_USART0((unsigned char*)prntbuf2);
#endif  
        
        
        
        prntbuf3[0] = 0;
#ifdef GPS_INFO_PRINT
         sprintf(prntbuf3,"%02d:%02d:%02d GPS=%c%d LAT=%+2.8f LON=%+3.8f HDG=%3.1f SPD=%3.1f ALT=%3.1f\r\n",
         ctime.hour, ctime.minute, ctime.secunde, gps.status, gps.SatCount, gps.Latitude, gps.Longitude,
         gps.Heading, gps.Speed, gps.Altitude);
         write_str_pdc_USART0((unsigned char*)prntbuf3);
#endif  
        
            

            
#ifdef MMC_CARD

          if( mmc_present != 0 )
          {
             if(gps.status == 'A')
             {            
                TPON
                if(ReturnPointSet == 0)
                {
                  ReturnLat = gps.Latitude*1000000; 
                  ReturnLon = gps.Longitude*1000000; 
                  ReturnPointSet = 1;
                }
             }
             else GPEON
               
             strcat(prntbuf,prntbuf2);   
             strcat(prntbuf,prntbuf3);   
             if(fileWrite(target, prntbuf, 512)  >= 0) 
              {
                TPOFF
                GPEOFF
              }             
           
          }

          //else ErrorPrint();

#endif
          
          
        }  
    cmd_rt = 0;
    
    }

}
}

/*void ErrorPrint(void)
{   
        char ebuf[25];
        extern int head_current;
        asm("nop "); 
        asm("ldr  r0,[PC,  #0x034]"); 
        asm("mov  r1, r14        "); 
        asm("str  r1, [r0, #0]   "); 
           
        sprintf(ebuf, "ERROR! dabtvec int R14 =0x%x\r\n", head_current);
        write_str_USART0((unsigned char*)ebuf);
        fileWrite(target, ebuf, 512);
        main();
}*/


void T0isr(void) 
{
 int status, tmp; 
 extern int xvalue, xdim[], xsum, xi;
 extern unsigned int msb;
 extern unsigned int lsb;
 extern int heading, ttc0;
// extern int flag;
*AT91C_AIC_ICCR = 0x1000;
ttc0 = 0;
tmp = *AT91C_TC0_SR;

// processing for X:
status = (unsigned)*AT91C_TC0_RA;
xvalue = status - 2300;
xdim[xi] = xvalue;

if(xi == 31) xi = 0;
else xi++;
xsum = xsum + xvalue - xdim[xi];
xvalue = xsum >> 5;

// processing for Y:

status = (unsigned)*AT91C_TC1_RA;
yvalue = status - 1850;
ydim[yi] = yvalue;
if(yi == 31) yi = 0;
else yi++;
ysum = ysum + yvalue - ydim[yi];
yvalue = ysum >> 5;

}

#ifdef FLYSKY

void T2isr(void) 
{
 int status;
 unsigned char a;  
 extern int tdim[], signal_sens;
// TPON
 *AT91C_AIC_ICCR = 0x4000; 
 status = *AT91C_TC2_SR;
 a = *AT91C_PIOA_PDSR >> 24;  
 //ToggleTp(); // for debug timer
  //if (ti == 0) GPEON
  //  else GPEOFF

   if(!( a & 0x72 ))
   {
    *AT91C_TC2_CCR = AT91C_TC_CLKDIS;   //disable clk  
    *AT91C_TC2_CCR = AT91C_TC_CLKEN;   //enable clk  
    signal_sens = 10;
   }
   if( a & 0x02 )
    {
      tdim[0]++;
    }
    else
    { 
      if(tdim[0] != 0)
      { 
        wdim[0] = 18*tdim[0]-500;  
        tdim[0] = 0;
      } 
    }
    if( a & 0x10 )
    {
      tdim[1]++;
    }
    else
    { 
      if(tdim[1] != 0)
      { 
        wdim[1] = 18*tdim[1]-500;  
        tdim[1] = 0;
      } 
    }
    if( a & 0x20 )
    {
      tdim[2]++;
    }
    else
    { 
      if(tdim[2] != 0)
      { 
        wdim[2] = 18*tdim[2]-500;  
        tdim[2] = 0;
      } 
    }
    if( a & 0x40 )
    {
      tdim[3]++;
    }
    else
    { 
      if(tdim[3] != 0)
      { 
        wdim[3] = 18*tdim[3]-500;  
        tdim[3] = 0;
      } 
    }
    
  //    TPOFF
}

#else


void T2isr(void) 
{
 int i, s, status;
 extern int tdim[10], ti, ttc2;
//GPEON
*AT91C_AIC_ICCR = 0x4000;
ttc2 = 0;
//status = *AT91C_TC2_SR;
if(*AT91C_TC2_SR & 0x10) return;
tdim[ti] = (unsigned)*AT91C_TC2_RA;
  if (ti >= NUMCHAN)
  {
//  tdim[NUMCHAN+1] = tdim[NUMCHAN+1] + tdim[ti];
//  if((tdim[NUMCHAN+2] >= 29000) && (tdim[NUMCHAN+2] <= 31000))
//   tdim[NUMCHAN+2] = tdim[NUMCHAN+1];   
//   tdim[NUMCHAN+1] = 0;
     ti = 0;
     if((tdim[NUMCHAN] >= 10000) && (tdim[NUMCHAN] <= 20000))
     {  
        for(i = 0, s = 0; i < NUMCHAN ; i++) 
        {
          if((tdim[i] >= 800) && (tdim[i] <= 3600))
          {
             wdim[i]=tdim[i]; s++;
          }
        } 
        if(s >= NUMCHAN) signal_sens = 10;
     }
     else 
     {
       ti=1;
     }
  }
  else  ti++;
  //GPEOFF
}

#endif

void PWMCisr(void) 
{
volatile int status; 
 extern int odim[], wi;
*AT91C_AIC_ICCR = 0x400;
status = *AT91C_PWMC_ISR;

while((*AT91C_PWMC_CH0_CCNTR == 0) || ((*AT91C_PWMC_CH0_CCNTR) == 1));

if ( wi == NUMCHAN )
{
//LEDON // For PWM testing..
//LEDOFF

*AT91C_PWMC_CH0_CUPDR = odim[NUMCHAN];
  wi = 0;
}
else
{
*AT91C_PWMC_CH0_CUPDR = odim[wi];
  wi++;
}

}

void SYSisr(void) 
{
int status;
*AT91C_AIC_ICCR = 0x0002;
status = *AT91C_RTTC_RTSR;
if ((status & 0x2) != 0) 
*AT91C_ADC_CR = 0x2;
cmd_rt=1;
  if(calibrate_sensors == 1) WDRESET

  if (indcnt++ == 5)
  {
    if(apilot_enable ==2) LEDON
  }
  else if(indcnt >= indicator)
  {
    LEDON
    indcnt=0;
  }
  else LEDOFF
    
if(++cmdcnt == 3)
  {
    cmdcnt = 0;  
    if(calibrate_sensors == 0) //HiPriorityFunc();
    *AT91C_AIC_ISCR = 0x80000000; //call soft-irq(irq31)
 //2222   *AT91C_AIC_ISCR = 0x40000000; //call soft-irq(irq0)
  }
}

void ADCisr(void) 
{
int status, tmp;
*AT91C_AIC_ICCR = 0x0010;
status = *AT91C_ADC_SR;
  if((status & 0x10) != 0)
  {
    tmp = *AT91C_ADC_CDR4;
    salt = salt - dalt[ialt] + tmp;
    dalt[ialt] = tmp;
    if(++ialt == ALT_SAMPLES) ialt = 0;
    malt = salt/ALT_SAMPLES; 
//    malt = *AT91C_ADC_CDR4;
  }
  else if((status & 0x20) != 0)
  {
    tmp = *AT91C_ADC_CDR5;
    sspeed = sspeed - dspeed[ispeed] + tmp;
    dspeed[ispeed] = tmp;
    if(++ispeed == SPEED_SAMPLES) ispeed = 0;
    mspeed = sspeed/SPEED_SAMPLES; 
//    mspeed = *AT91C_ADC_CDR5;
  }
  else if((status & 0x40) != 0)
  {
    tmp = *AT91C_ADC_CDR6;
    sgyro = sgyro - dgyro[igyro] + tmp;
    dgyro[igyro] = tmp;
    if(++igyro == GYRO_SAMPLES)
    {
      igyro = 0;
    }
    mgyro = sgyro/GYRO_SAMPLES; 
  }
    else if((status & 0x08) != 0)
  {
    v_akk = *AT91C_ADC_CDR3;
  }
  
}
#ifdef MMC_CARD

int ReadParam( tFILE * f, FSSc * st)
{
  int pnum, m, k; 
  char rdbuf[100], buf[32], *endptr[10];

    while(1)
    {
        if(fileRead(f, rdbuf, 100)>0)
        {
          if(rdbuf[0] == 'P')  
          {
            pnum = 0;
            m=0;
            for (k=1; k<=strlen(rdbuf); k++)
            {
              if(rdbuf[k] == ',')
               { 
                  buf[m] =0x00;
                  switch(pnum)
                  {
                    case 0 :  st->Kst_roll   = strtod(buf, endptr); break;
                    case 1 :  st->Kast_roll  = strtod(buf, endptr); break;
                    case 2 :  st->Kfar_roll  = strtod(buf, endptr); break;
                    case 3 :  st->Kst_pitch  = strtod(buf, endptr); break;
                    case 4 :  st->Kast_pitch = strtod(buf, endptr); break;
                    case 5 :  st->Kfar_pitch = strtod(buf, endptr); break;
                    case 6 :  st->Kst_speed  = strtod(buf, endptr); break;
                    case 7 :  st->Kast_speed = strtod(buf, endptr); break;
                    case 8 :  st->Kfar_speed = strtod(buf, endptr); break;
                    case 9 :  st->Kst_hdg    = strtod(buf, endptr); break;
                    case 10 : st->Kast_hdg   = strtod(buf, endptr); break;
                    case 11 : st->Kst_alt    = strtod(buf, endptr); break;
                    case 12 : st->Kast_alt   = strtod(buf, endptr); break;
                    default: break;
                   }
                      m=0;
                      pnum++;
               }
               else
               {
                 buf[m] = rdbuf[k];
                 m++;
               }
            }
          break;  
          }
        }
        else return -1;
          
    }     
return(1); 
}
/*int GetNextPoint( tFILE * f, WPS * wp)
{
  int pnum, m, k; 
  char rdbuf[100], buf[32], *endptr[10];

    while(1)
    {
        if(fileRead(f, rdbuf, 100)>0)
        {
          if(rdbuf[0] == 'M')  
          {
            pnum = 0;
            m=0;
            for (k=1; k<=strlen(rdbuf); k++)
            {
              if(rdbuf[k] == ',')
              {  
                 buf[m] =0x00;
                 switch(pnum)
                 {

                  case 0 :  wp->point_num = strtod(buf, endptr); break;
                  case 1 :  wp->alt_requested = strtod(buf, endptr); break;
                  case 2 :  wp->head_requested = strtod(buf, endptr); break;
                  case 3 :  wp->speed_requested = strtod(buf, endptr); break;
                  case 4 :  wp->time_requested = strtod(buf, endptr); break;
                  case 5 :  if(buf[0] == 'N')
                            {
                              wp->next_point = wp->point_num+1;
                              break;
                            }
                            else
                            {
                              fileReset(f);
                              wp->next_point =strtod(buf, endptr);
                              break;
                            }
                  default: break;
                 }
                 m=0;
                 pnum++;
                 if(wp->next_point!=wp->point_num) break;  
              }
              else
              {
                buf[m] = rdbuf[k];
                m++;
              }
            }
          }
         if(pnum == 6) break;
        }
        else return -1;
          
    }     
return(0); 
}*/
int GetGPSNextPoint( tFILE * f, WGPS * wp)
{
  int pnum, m, k; 
  char rdbuf[100], buf[32], *endptr[10];
  

    while(1)
    {
        if(fileRead(f, rdbuf, 100)>0)
        {
          if(rdbuf[0] == 'G')  
          {
            pnum = 0;
            m=0;
            for (k=1; k<=strlen(rdbuf); k++)
            {
              if((rdbuf[k] == ',') || (rdbuf[k] == 0x0A) || (rdbuf[k] == 0x0D)) 
              {  
                 buf[m] =0;
                 switch(pnum)
                 {
 /*struct WGPS gway= { id, lat, lon, alt, spd, crs, dur, npt };*/

                  case 0 :  wp->id = strtod(buf, endptr); break;
                  case 1 :  wp->lat = strtod(buf, endptr); break;
                  case 2 :  wp->lon = strtod(buf, endptr); break;
                  case 3 :  wp->alt = strtod(buf, endptr); break;
                  case 4 :  wp->spd = strtod(buf, endptr); break;
                  case 5 :  wp->crs = strtod(buf, endptr); break;
                  case 6 :  wp->dur = strtod(buf, endptr); break;
                  case 7 :  if(buf[0] == 'N')
                            {
                              wp->npt = wp->id+1;
                            }
                            else 
                            {
                              wp->npt = strtod(buf, endptr);
                            }
                  default: break;
                 }
                 m=0;
                 if(pnum++ == 7)
                   return(0); 
              }
              else
              {
                buf[m] = rdbuf[k];
                buf[m+1] = 0;
                m++;
              }
            }
          }
        }
        else return -1;
    }     
}


#endif

void ToggleTp( void )
{
      if(flag == 0)
      {
        flag = 1;
        TPON
//        LEDON
      }
      else
      {
        flag =0;
        TPOFF
//        LEDOFF
      }
}


void FlStSys(FSSi *in, FSSo *out, FSSp *keep, FSSc *ctrl, TEST *tst)
{
  int roll_current, pitch_current, roll_var, pitch_var;
  int speed_current, r, p,t, a,b,c;


// *************************** Roll control **********************************
roll_dim[ri]=roll_current= in->roll-keep->roll;
pitch_dim[ri]=pitch_current= in->pitch-keep->pitch;
ri++;
if ( ri >= FSS_SAMPLES+1) ri=0;
roll_sum = roll_sum + roll_current-roll_dim[ri];
roll_var = roll_sum - roll_pr;
roll_long_sum = roll_long_sum + roll_current - roll_long_sum/FAR_SAMPLES;
roll_pr=roll_sum;

a = ctrl->Kst_roll*((float)roll_sum/FSS_SAMPLES);
b = ctrl->Kast_roll*((float)roll_var/FSS_SAMPLES);
c = ctrl->Kfar_roll*((float)roll_long_sum/FAR_SAMPLES);

r =-a+b-c;

#ifdef DRAGON_SPORTSTER      
if(r >= 100) r=100;
 else if(r <= -100) r = -100;
 if(apilot_enable == 2) 
 {
   out->ailerons = r;
   out->rudder = r/4;
 }
 else
 {
   out->ailerons = r/2;
   out->rudder = r; 
 }
#elif  DUSTY  1
 if(r >= 80) r=80;
 else if(r <= -80) r = -80;
 
 
 if(apilot_enable == 2) 
 {
   out->ailerons = r;
   out->rudder = r/4;
 }
 else
 {
   out->ailerons = r/4;
   out->rudder = r; 
 }
#else        
if(r >= 100) r=100;
 else if(r <= -100) r = -100;
 if(apilot_enable == 2) 
 {
   out->ailerons = r;
   out->rudder = r/4; 
 }
 else
 {
   out->ailerons = r/2;
   out->rudder = r; 
 }
#endif        


 

/*
 if(r >= 100) r=100;
 else if(r <= -100) r = -100;

    out->ailerons = r;
    out->rudder = r; //temporary 1:1

*/
//}
// ************************** Pitch control *********************************

//pitch_dim2[pi]=pitch_current= in->pitch-keep->pitch;

pitch_sum = pitch_sum + pitch_current-pitch_dim[ri];
pitch_var = pitch_sum - pitch_pr;
pitch_long_sum = pitch_long_sum + pitch_current - pitch_long_sum/FAR_SAMPLES;
pitch_pr=pitch_sum;

a = ctrl->Kst_pitch*((float)pitch_sum/FSS_SAMPLES);
b = ctrl->Kast_pitch*((float)pitch_var/FSS_SAMPLES);
c = ctrl->Kfar_pitch*((float)pitch_long_sum/FAR_SAMPLES);

p =-a+b-c;



#ifdef DRAGON_SPORTSTER      
if(p >= 100) p=100;
else if(p <= -100) p = -100;
#elif  DUSTY  1
if(p >= 80) p=80;
else if(p <= -80) p = -80;
#else        
if(p >= 100) p=100;
else if(p <= -100) p = -100;
#endif        

out->elevator =  p;


// ************************** Throttle control **********************************

speed_current = in->speed - keep->speed;
speed_sum = speed_sum + speed_current -  speed_sum/SPEED_SAMPLES_HPF;

if(++speed_var_cnt == 9)
  {
    speed_var_cnt = 0;
    speed_var = in->speed - speed_pr;
    speed_pr = in->speed;
  }

tst->p1 = speed_sum/SPEED_SAMPLES_HPF;
tst->p2 = speed_var;
tst->p3 = speed_trim;

if(((speed_sum/SPEED_SAMPLES_HPF) > 2) && (speed_trim > -30))  speed_trim = speed_trim - ctrl->Kfar_speed;
else if(((speed_sum/SPEED_SAMPLES_HPF) < -2) && (speed_trim < +30)) speed_trim = speed_trim + ctrl->Kfar_speed;


a = ctrl->Kst_speed*((float)speed_sum/SPEED_SAMPLES_HPF);

b = ctrl->Kast_speed*(float)speed_var;

c = speed_trim;


t = 127 - a - b + c; 


//tst->p1 = a;
//tst->p2 = b;
//tst->p3 = c;

//  t = 127-ctrl->Kst_speed*((float)speed_sum/SPEED_SAMPLES_HPF)+ctrl->Kast_speed*(float)speed_var + speed_trim;
//  t = 127-ctrl->Kst_speed*((float)speed_sum/SPEED_SAMPLES_HPF) + speed_trim;

  
  tst->p4 = t;


if(t >= 250) t=250;
else if(t <= TROTTLE_XX) t = TROTTLE_XX;
out->throttle = t;


}


void HiPriorityFunc(void)
{
  int i;
  *AT91C_AIC_ICCR = 0x80000000;
//2222  *AT91C_AIC_ICCR = 0x40000000;
//        vgyro =((mgyro-vgyro_0)*12)/10;//(o/s)*10   //  Q=N*(3072mV/1024)/25mV/o/sek
/*        The meassurement is shown that 1.2 error. Therefore we delete 12/10...*/
        vgyro =vgyro_0-mgyro;//(degree per second)*10   //  Q=N*(3072mV/1024)/25mV/o/sek

 // *AT91C_AIC_EOICR = 0x80000000;

      if (g_nd == 1 ) 
      {
        g_nd = 0;
        i = GPSProcessing();              
        if(i !=0 )
        {
          sprintf( errbuf, "\r\nGPS Error %d\r\n", i );
          //write_str_pdc_USART0((unsigned char*)errbuf);
        }
      }
        
//               if(gps.status == 'A')
               if((gps.Speed >= GPS_MIN_SPEED) && (gps.status == 'A'))
                 gnd_speed = gps.Speed*1.852;  //speed for roll calculating 
               else
                 gnd_speed = in.speed;//*5.2; // / 
        

               
//              sprintf( errbuf, "%d:", vgyro);
//              write_str_USART0((unsigned char*)errbuf);
               
               
#ifdef AUTO_ROLL
              if (gnd_speed > 10)
              {
                teta = atan(((((float)vgyro*PI)/180)*(gnd_speed/3.6))/98); //roll calculating, 
                //because vgyro in grad*10
              }
              else
              {
                teta =asin((float)yvalue/500);  //static correction roll rad 
              }
#else
#ifdef ROLL_BY_GYRO           
              if(gnd_speed <= GPS_MIN_SPEED) gnd_speed = GPS_MIN_SPEED;  //limit minimum gnd speed
              teta = atan(((((float)vgyro*PI)/180)*(gnd_speed/3.6))/98); //roll calculating  
#else            
#ifdef NO_SIN              
               teta =(float)yvalue/500;  //static correction roll rad 
#else              
               teta =asin((float)yvalue/500);  //static correction roll rad 
#endif               
#endif              
#endif              
              
              
#ifdef NO_SIN              
               fi = -(float)xvalue/500;
               ateta =(int)(((float)yvalue/500)*180/PI);  //teta from accelerometer
               
#else
               fi = -asin((float)xvalue/500);
               ateta =(int)((asin((float)yvalue/500))*180/PI);  //teta from accelerometer 
#endif               
          
//              in.roll  = (int)((teta*180)/PI);
//              in.pitch = (int)((fi*180)/PI);
                in.roll  = (int)((teta*1800)/PI); //roll*10 in degree 
                in.pitch = (int)((fi*1800)/PI);   //pitch*10 
                in.altitude =  malt - valt_0;            //   
                
#ifdef ROLL_DEBUG                
                in.speed =30;      // / 
#else

                in.speed =7.28*sqrt(abs(vspeed_0 -  mspeed));//*5.2;      // / 
#endif

                
                FlStSys(&in, &out, &keep, &ctrl, &tst); 

       if(signal_sens > 5)   //analysing for transmitter OK
       {
         in_signal = 1;
         signal_sens--;
       }
       else
       {
         in_signal = 0;
       }

       if(apilot_enable != 0)
       {

         
#ifdef ONLY_THROTTLE
         
  #ifdef DRAGON_SPORTSTER      
        odim[0] = WMED - (out.ailerons*WAMP)/128;
        odim[1] = WMED - (out.elevator*WAMP)/128;   
//        odim[2] = WMED + ((out.throttle-127)*WAMP)/128;   
        odim[3] = WMED - (out.rudder*WAMP)/128;
        odim[4] = odim[5] = WMED;   
  #elif  DUSTY  1
        odim[0] = WMED - (out.ailerons*WAMP)/128;       
        odim[1] = WMED + (out.elevator*WAMP)/128;   
//        odim[2] = WMED + ((out.throttle-127)*WAMP)/128;   
        odim[3] = WMED - ((out.rudder)*WAMP)/128;  
        odim[4] = odim[5] = odim[6]=odim[7]=WMED;   
  #else        
        odim[0] = WMED + (out.ailerons*WAMP)/128;
        odim[1] = WMED + (out.elevator*WAMP)/128;   
//        odim[2] = WMED + ((out.throttle-127)*WAMP)/128;   
        odim[3] = WMED + (out.rudder*WAMP)/128;   
        odim[4] = odim[5] = odim[6]=odim[7]=WMED;   
  #endif 
        
        if(in_signal == 1)  odim[2]=wdim[2];
        else odim[2]=WMIN;
        
       }
         
#else
         
  #ifdef DRAGON_SPORTSTER      
        odim[0] = WMED - (out.ailerons*WAMP)/128;
        odim[1] = WMED - (out.elevator*WAMP)/128;   
        odim[2] = WMED + ((out.throttle-127)*WAMP)/128;   
        odim[3] = WMED - (out.rudder*WAMP)/128;
        odim[4] = odim[5] = WMED;   
  #elif  DUSTY  1
        odim[0] = WMED - (out.ailerons*WAMP)/128;       
        odim[1] = WMED + (out.elevator*WAMP)/128;   
        odim[2] = WMED + ((out.throttle-127)*WAMP)/128;   
        odim[3] = WMED - ((out.rudder)*WAMP)/128;  
        odim[4] = odim[5] = odim[6]=odim[7]=WMED;   
  #else        
        odim[0] = WMED + (out.ailerons*WAMP)/128;
        odim[1] = WMED + (out.elevator*WAMP)/128;   
        odim[2] = WMED + ((out.throttle-127)*WAMP)/128;   
        odim[3] = WMED + (out.rudder*WAMP)/128;   
        odim[4] = odim[5] = odim[6]=odim[7]=WMED;   
  #endif        
       }
       else 
       {
         if(in_signal == 1)
         { 
           for(i=4; i < NUMCHAN; i++) odim[i]=wdim[i];
           odim[0]=(wdim[0]-WMED)/2+WMED;
           odim[1]=(wdim[1]-WMED)/2+WMED;
//           odim[2]=wdim[2];
           odim[2]=((wdim[2]-WMED)*14)/10+WMED;
           odim[3]=(wdim[3]-WMED)/2+WMED;
         }
         else
         {
          apilot_enable = 2;   //emergency switching to AP
          roll_long_sum=0;     //reset lohg summes
          pitch_long_sum=0;    //reset lohg summes
          indicator = 64;
         }
       }
#endif       
       

#ifdef DRAGON_SPORTSTER
        servo.ailerons = -((odim[0]-WMED)*128)/WAMP;
        servo.elevator = ((odim[1]-WMED)*128)/WAMP; //inverting for correct logging
        servo.throttle = (((odim[2]*128)-WMED)/WAMP)-50;
        servo.rudder   = -((odim[3]-WMED)*128)/WAMP; //inverting for correct logging
#elif  DUSTY  1
        servo.ailerons = -((odim[0]-WMED)*128)/WAMP;
        servo.elevator = ((odim[1]-WMED)*128)/WAMP; //inverting for correct logging
        servo.throttle = (((odim[2]*128)-WMED)/WAMP)-50;
        servo.rudder   = -((odim[3]-WMED)*128)/WAMP; //inverting for correct logging
#else        
        servo.ailerons = -((odim[0]-WMED)*128)/WAMP;
        servo.elevator = ((odim[1]-WMED)*128)/WAMP; //inverting for correct logging
        servo.throttle = (((odim[2]*128)-WMED)/WAMP)-50;
        servo.rudder   = ((odim[3]-WMED)*128)/WAMP; //inverting for correct logging
#endif        
}
