#include "include/AT91SAM7S256.h"
#include "twi.h"

#define LEDON   *AT91C_PIOA_CODR = 0x100;  //set reg to 0 (led on)
#define LEDOFF  *AT91C_PIOA_SODR = 0x100;  //set reg to 1 (led off)
#define TPOFF    *AT91C_PIOA_CODR = 0x080;  //set reg to 0 
#define TPON     *AT91C_PIOA_SODR = 0x080;  //set reg to 1 


unsigned RxCnt;
unsigned MsbByte;
unsigned LsbByte;
unsigned I2C_busy;
//unsigned stat;
unsigned TMPSTAT;

//*=========================================================
//*		INIT
//*=========================================================
void TwiInit(void)
{
  // pin definition for TWI  
  *AT91C_PIOA_PDR = BIT3 | BIT4;
  *AT91C_PIOA_ASR = BIT3 | BIT4;
  *AT91C_PIOA_BSR = 0;  
  //set int vector for TWI  
//  AT91C_AIC_SVR[AT91C_ID_TWI]   = (unsigned)TWIisr;  
  // Set edge trigger for TWI interrupts
//  AT91C_AIC_SMR[AT91C_ID_TWI]  = AT91C_AIC_PRIOR_LOWEST
//                                 | AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED;
//AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE  Internal Sources Level Sensitive
//AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED   Internal Sources Edge triggered
//AT91C_AIC_SRCTYPE_EXT_HIGH_LEVEL       External Sources High-level Sensitive
//AT91C_AIC_SRCTYPE_EXT_POSITIVE_EDGE    External Sources Positive Edge triggere
  // enable twi clock
  *AT91C_PMC_PCER = 1<<AT91C_ID_TWI;
  // Software reset
  *AT91C_TWI_CR = AT91C_TWI_SWRST;
  //enable interrupt for TWI
  *AT91C_AIC_IECR = 1<<AT91C_ID_TWI;
  //set clicks for twi
  //MCK = 48 MHz T_mck=1/48=20.83333 ns   
  //T_twi = ((CLDIV * 2^CKDIV)+3)*T_mck 
  *AT91C_TWI_CWGR =(5<<16) | (8<<8) | 8; 
  // Set edge trigger for TWI interrupts
  *AT91C_TWI_IER = AT91C_TWI_RXRDY | AT91C_TWI_TXCOMP;
//  *AT91C_TWI_IER = AT91C_TWI_TXCOMP | AT91C_TWI_RXRDY | AT91C_TWI_NACK;
//  *AT91C_TWI_IER =  AT91C_TWI_RXRDY |  AT91C_TWI_TXRDY;
//                    AT91C_TWI_TXCOMP | AT91C_TWI_NACK;
  *AT91C_TWI_CR = AT91C_TWI_MSEN;
  I2C_busy=0;
}

/*
// Read from I2C device without interrupts
short int TwiRead(unsigned slave_address,
                 unsigned internal_address,
                 unsigned addrsize)
{ 
//  while( (*AT91C_TWI_SR & AT91C_TWI_TXCOMP)==0);
  if(addrsize != 0)
  {
    *AT91C_TWI_IADR = internal_address;
  }
  *AT91C_TWI_MMR = (slave_address << 16) | (1 << 12) | (addrsize << 8);
  *AT91C_TWI_CR = AT91C_TWI_START;
   while((*AT91C_TWI_SR & AT91C_TWI_RXRDY)== 0);
   MsbByte=*AT91C_TWI_RHR;
   *AT91C_TWI_CR = AT91C_TWI_STOP;
   while((*AT91C_TWI_SR & AT91C_TWI_RXRDY)== 0);
   LsbByte=*AT91C_TWI_RHR;
  return (LsbByte|(MsbByte<<8));
}


// Write to I2C device without interrupts
void TwiWrite(unsigned slave_address,
              unsigned internal_address,
              unsigned addrsize,
              unsigned data)
{
 // while( (*AT91C_TWI_SR & AT91C_TWI_TXCOMP)==0);
  if(addrsize != 0)
  {
    *AT91C_TWI_IADR = internal_address;
  }
  *AT91C_TWI_MMR = (slave_address << 16) | (addrsize << 8);
  *AT91C_TWI_THR = data;
  *AT91C_TWI_CR = AT91C_TWI_STOP;
  return;
}

*/

// Read from I2C device with interrupts
short int TwiRead(unsigned slave_address,
                 unsigned internal_address,
                 unsigned addrsize)
{ 
  while(I2C_busy != 0 );
  I2C_busy = 1;
  if(addrsize != 0)
  {
    *AT91C_TWI_IADR = internal_address;
  }
  *AT91C_TWI_MMR = (slave_address << 16) | (1 << 12) | (addrsize << 8);
  RxCnt = 0;
  *AT91C_TWI_CR = AT91C_TWI_START;
  return (LsbByte|(MsbByte<<8));
}

// Write to I2C device without interrupts
void TwiWrite(unsigned slave_address,
              unsigned internal_address,
              unsigned addrsize,
              unsigned data)
{
  while(I2C_busy != 0 );
  I2C_busy = 2;
  if(addrsize != 0)
  {
    *AT91C_TWI_IADR = internal_address;
  }
  *AT91C_TWI_MMR = (slave_address << 16) | (addrsize << 8);
  *AT91C_TWI_THR = data;
  *AT91C_TWI_CR = AT91C_TWI_STOP;
  return;
}


// Interrupt handler for I2C device
void TWIisr(void) 
{
  int status;
  status = *AT91C_TWI_SR;
  TMPSTAT = status;
  if ((status & AT91C_TWI_RXRDY)!= 0)
  {
    switch ( RxCnt )
    {
    case 0:  
     MsbByte=*AT91C_TWI_RHR;
     *AT91C_TWI_CR = AT91C_TWI_STOP;
     RxCnt = 1;
    break;
    case 1:  
     LsbByte=*AT91C_TWI_RHR; 
     RxCnt = 2;
     I2C_busy = 0;
     break;
    default: break;
    }
  }
  if ((status & AT91C_TWI_TXCOMP)!= 0)
  {
     I2C_busy = 0;
  }
  *AT91C_AIC_ICCR = 1<<AT91C_ID_TWI;
}
